@@ -30,13 +30,10 @@ import os
 import breezy
 from ...commands import plugin_cmds
 from ...hooks import install_lazy_named_hook
-from ...sixish import (
-    viewitems,
-    )
 from ... import trace
 
 from .info import (
-    bzr_plugin_version as version_info,
+    brz_plugin_version as version_info,
     )
 
 
@@ -58,26 +55,16 @@ commands = {
         "debrelease": [],
         }
 
-for command, aliases in viewitems(commands):
+for command, aliases in commands.items():
     plugin_cmds.register_lazy(
         'cmd_' + command, aliases, __name__ + ".cmds")
 
-builddeb_dir = '.bzr-builddeb'
-default_conf = os.path.join(builddeb_dir, 'default.conf')
-
 
 def global_conf():
-    try:
-        from ...bedding import config_dir
-    except ImportError:
-        from ...config import config_dir
+    from ...bedding import config_dir
     return os.path.join(config_dir(), 'builddeb.conf')
 
 
-local_conf = os.path.join(builddeb_dir, 'local.conf')
-new_local_conf = 'debian/local.conf.local'
-new_conf = 'debian/bzr-builddeb.conf'
-
 default_build_dir = '../build-area'
 default_orig_dir = '..'
 default_result_dir = '..'
@@ -125,7 +112,8 @@ else:
     register_lazy(
         "breezy.directory", "directories", "apt:",
         __name__ + '.directory', 'VcsDirectory',
-        help="Directory that uses Debian Vcs-* control fields to look up branches")
+        help="Directory that uses Debian Vcs-* control "
+             "fields to look up branches")
     register_lazy(
         "breezy.directory", "AliasDirectory.branch_aliases", "upstream",
         __name__ + ".directory", "upstream_branch_alias",
@@ -194,10 +182,12 @@ def debian_tag_name(branch, revid):
     from .util import (
         debuild_config, find_changelog, MissingChangelogError)
     t = branch.repository.revision_tree(revid)
-    config = debuild_config(t)
+    subpath = ''
+    config = debuild_config(t, subpath=subpath)
     try:
         (changelog, top_level) = find_changelog(
-            t, config.build_type == BUILD_TYPE_MERGE)
+            t, subpath=subpath,
+            merge=(config.build_type == BUILD_TYPE_MERGE))
     except MissingChangelogError:
         # Not a debian package
         return None
@@ -220,14 +210,16 @@ def pre_merge_fix_ancestry(merger):
         return
     if getattr(merger, "other_branch", None) is None:
         return
+    # This only works for packages that live in the root. That seems fine,
+    # though?
     if (not merger.this_tree.is_versioned("debian/changelog") or
             not merger.other_tree.is_versioned("debian/changelog")):
         return
-    this_config = debuild_config(merger.this_tree)
-    other_config = debuild_config(merger.other_tree)
+    this_config = debuild_config(merger.this_tree, '')
+    other_config = debuild_config(merger.other_tree, '')
     if not (this_config.build_type == BUILD_TYPE_NATIVE or
             other_config.build_type == BUILD_TYPE_NATIVE):
-        from .errors import PackageVersionNotPresent
+        from .upstream import PackageVersionNotPresent
         try:
             fix_ancestry_as_needed(
                 merger.this_tree, merger.other_branch,
@@ -259,12 +251,7 @@ install_lazy_named_hook(
     'Debian ancestry fixing')
 
 
-from . import quilt
-
-
 def load_tests(loader, basic_tests, pattern):
     basic_tests.addTest(
         loader.loadTestsFromModuleNames([__name__ + '.tests']))
-    basic_tests.addTest(
-        loader.loadTestsFromModuleNames([__name__ + '.quilt.tests']))
     return basic_tests
