@@ -61,6 +61,8 @@ class SvnBuildPackageMappedConfig(object):
 class UpstreamMetadataSyntaxError(BzrError):
     """There is a syntax error in the debian/upstream/metadata file."""
 
+    _fmt = 'Unable to parse upstream metadata file %(path)s: %(error)s'
+
     def __init__(self, path, error):
         self.path = path
         self.error = error
@@ -75,8 +77,15 @@ class UpstreamMetadataConfig(object):
     def __init__(self, text):
         try:
             self.metadata = yaml.safe_load(text)
-        except yaml.scanner.ScannerError as e:
+        except (yaml.scanner.ScannerError, yaml.composer.ComposerError,
+                yaml.parser.ParserError) as e:
             raise UpstreamMetadataSyntaxError('debian/upstream/metadata', e)
+        if isinstance(self.metadata, str):
+            raise UpstreamMetadataSyntaxError(
+              'debian/upstream/metadata', TypeError(self.metadata))
+        if isinstance(self.metadata, list):
+            raise UpstreamMetadataSyntaxError(
+              'debian/upstream/metadata', TypeError(self.metadata))
 
     def get_value(self, section, option):
         if section == "BUILDDEB":
@@ -332,10 +341,6 @@ class DebBuildConfig(object):
 
     merge = _bool_property('merge', "Run in merge mode")
 
-    debug_pristine_tar = _bool_property(
-      'debug-pristine-tar',
-      "Save some context when pristine-tar fails")
-
     @property
     def build_type(self):
         if self.merge:
@@ -369,5 +374,3 @@ def _test():
 
 if __name__ == '__main__':
     _test()
-
-# vim: ts=2 sts=2 sw=2
