@@ -20,8 +20,12 @@
 
 from __future__ import absolute_import
 
+try:
+    from ...revisionspec import InvalidRevisionSpec
+except ImportError:  # breezy < 3.2
+    from ...errors import InvalidRevisionSpec
+
 from ...errors import (
-    InvalidRevisionSpec,
     NoSuchTag,
     )
 from ...revisionspec import RevisionSpec, RevisionInfo
@@ -46,7 +50,7 @@ class RevisionSpec_package(RevisionSpec):
 
     def _match_on(self, branch, revs=None):
         version_spec = self.spec
-        dist_spec = None
+        dist_spec = None  # noqa: F841
 
         if version_spec == '':
             raise VersionNotSpecified
@@ -77,18 +81,21 @@ class RevisionSpec_upstream(RevisionSpec):
             find_changelog,
             MissingChangelogError,
             )
-        from .upstream.pristinetar import PristineTarSource
+        from .upstream.pristinetar import get_pristine_tar_source
+        from .upstream import StackedUpstreamSource
         from debian.changelog import Version
-        tree = WorkingTree.open_containing('.')[0]
+        tree, subpath = WorkingTree.open_containing('.')
         try:
-            (cl, top_level) = find_changelog(tree, False)
+            (cl, top_level) = find_changelog(tree, subpath, merge=False)
         except MissingChangelogError as e:
-            raise InvalidRevisionSpec(self.user_spec, branch,
+            raise InvalidRevisionSpec(
+                self.user_spec, branch,
                 "no debian/changelog file found: %s" % e)
         if self.spec == '':
             version_spec = cl.version.upstream_version
             if not cl.version.debian_version:
-                raise InvalidRevisionSpec(self.user_spec, branch,
+                raise InvalidRevisionSpec(
+                    self.user_spec, branch,
                     "This is a native package.")
         else:
             version = Version(self.spec)
@@ -97,10 +104,12 @@ class RevisionSpec_upstream(RevisionSpec):
             else:
                 version_spec = self.spec
 
-        pristine_tar_source = PristineTarSource(branch)
+        upstream_source = StackedUpstreamSource([
+            get_pristine_tar_source(tree, branch),
+            ])
         try:
-            revision_id = pristine_tar_source.version_as_revisions(cl.package,
-                version_spec)[None]
+            revision_id = upstream_source.version_as_revisions(
+                cl.package, version_spec)[None]
             return RevisionInfo.from_revision_id(
                 branch, revision_id)
         except NoSuchTag:
