@@ -33,6 +33,8 @@ import tempfile
 import tarfile
 import zipfile
 
+from unittest import expectedFailure
+
 from ....revision import (
     NULL_REVISION,
     Revision,
@@ -48,12 +50,7 @@ from ....tests.features import (
 from ..config import (
     DebBuildConfig,
     )
-from ..errors import (
-    MissingUpstreamTarball,
-    PackageVersionNotPresent,
-    )
 
-from ....sixish import text_type
 from . import (
     LzmaFeature,
     XzFeature,
@@ -61,13 +58,13 @@ from . import (
     TestCaseWithTransport,
     )
 from ..upstream import (
+    MissingUpstreamTarball,
+    PackageVersionNotPresent,
     AptSource,
     StackedUpstreamSource,
     TarfileSource,
     UpstreamProvider,
     UpstreamSource,
-    UScanSource,
-    UScanError,
     extract_tarball_version,
     gather_orig_files,
     new_tarball_name,
@@ -86,15 +83,21 @@ from ..upstream.branch import (
     upstream_version_add_revision
     )
 from ..upstream.pristinetar import (
-    PristineTarSource,
+    get_pristine_tar_source,
     is_upstream_tag,
     revision_pristine_tar_format,
     revision_pristine_tar_delta,
     upstream_tag_version,
+    GitPristineTarSource,
+    BzrPristineTarSource,
+    )
+from ..upstream.uscan import (
+    UScanSource,
+    UScanError,
     )
 
 
-svn_plugin = PluginLoadedFeature('svn')
+svn_plugin = ModuleAvailableFeature('breezy.plugins.svn.mapping')
 dulwich = ModuleAvailableFeature('dulwich')
 
 
@@ -161,9 +164,10 @@ class MockAptCaller(object):
 class AptSourceTests(TestCase):
 
     def test_get_apt_command_for_source(self):
-        self.assertEqual("apt-get source -y --only-source --tar-only "
-                "apackage=someversion",
-                AptSource()._get_command("apackage", "someversion"))
+        self.assertEqual(
+            "apt-get source -y --only-source --tar-only "
+            "apackage=someversion",
+            AptSource()._get_command("apackage", "someversion"))
 
     def test_apt_provider_no_package(self):
         caller = MockAptCaller()
@@ -171,7 +175,8 @@ class AptSourceTests(TestCase):
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
+        self.assertRaises(
+            PackageVersionNotPresent, src.fetch_tarballs,
             "apackage", "0.2", "target", _apt_pkg=apt_pkg)
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
@@ -182,12 +187,14 @@ class AptSourceTests(TestCase):
 
     def test_apt_provider_wrong_version(self):
         caller = MockAptCaller()
-        sources = MockSources(["0.1-1"],
+        sources = MockSources(
+            ["0.1-1"],
             [[("checksum", 0, "apackage_0.1.orig.tar.gz", "tar")]])
         apt_pkg = MockAptPkg(sources)
         src = AptSource()
         src._run_apt_source = caller.call
-        self.assertRaises(PackageVersionNotPresent, src.fetch_tarballs,
+        self.assertRaises(
+            PackageVersionNotPresent, src.fetch_tarballs,
             "apackage", "0.2", "target", _apt_pkg=apt_pkg)
         self.assertEqual(1, apt_pkg.init_called_times)
         self.assertEqual(1, apt_pkg.get_pkg_source_records_called_times)
@@ -299,14 +306,25 @@ class AptSourceTests(TestCase):
 
 class RecordingSource(UpstreamSource):
 
-    def __init__(self, succeed, latest=None):
+    def __init__(self, succeed, latest=None, recent=None):
         self._succeed = succeed
         self._specific_versions = []
+        if recent:
+            recent.sort()
+        if latest is None and recent:
+            latest = recent[-1]
         self._latest = latest
+        if recent is None and latest:
+            recent = [latest]
+        self._recent = recent
 
     def get_latest_version(self, package, current_version):
         return self._latest
 
+    def get_recent_versions(self, package, since_version=None):
+        return [v for v in self._recent
+                if since_version is None or v > since_version]
+
     def fetch_tarballs(self, package, version, target_dir, components=None):
         self._specific_versions.append((package, version, target_dir))
         if not self._succeed:
@@ -335,11 +353,24 @@ class StackedUpstreamSourceTests(TestCase):
         stack = StackedUpstreamSource([a, b])
         self.assertEquals("1.1", stack.get_latest_version("mypkg", "1.0"))
 
+    def test_get_recent_versions(self):
+        a = RecordingSource(False, recent=["1.0", "1.1"])
+        b = RecordingSource(False, recent=["1.1", "1.2"])
+        stack = StackedUpstreamSource([a, b])
+        self.assertEquals(
+            ["1.0", "1.1", "1.2"], stack.get_recent_versions("mypkg", "0.9"))
+        self.assertEquals(
+            ["1.0", "1.1", "1.2"], stack.get_recent_versions("mypkg"))
+        self.assertEquals(
+            ["1.2"], stack.get_recent_versions("mypkg", "1.1"))
+
     def test_repr(self):
-        self.assertEquals("StackedUpstreamSource([])",
-                repr(StackedUpstreamSource([])))
-        self.assertEquals("StackedUpstreamSource([RecordingSource()])",
-                repr(StackedUpstreamSource([RecordingSource(False)])))
+        self.assertEquals(
+            "StackedUpstreamSource([])",
+            repr(StackedUpstreamSource([])))
+        self.assertEquals(
+            "StackedUpstreamSource([RecordingSource()])",
+            repr(StackedUpstreamSource([RecordingSource(False)])))
 
     def test_none(self):
         a = RecordingSource(False)
@@ -351,91 +382,6 @@ class StackedUpstreamSourceTests(TestCase):
         self.assertEquals([("pkg", "1.0", "bla")], a._specific_versions)
 
 
-class UScanSourceTests(TestCaseWithTransport):
-
-    def setUp(self):
-        super(UScanSourceTests, self).setUp()
-        self.tree = self.make_branch_and_tree('.')
-
-    def test_export_watchfile_none(self):
-        src = UScanSource(self.tree, False)
-        self.assertRaises(NoSuchFile, src._export_file, 'watch', self.test_dir)
-
-    def test_export_watchfile_top_level(self):
-        src = UScanSource(self.tree, True)
-        self.build_tree(['watch'])
-        self.tree.add(['watch'])
-        tmpdir = tempfile.mkdtemp()
-        try:
-            self.assertIsNot(src._export_file('watch', tmpdir), None)
-        finally:
-            shutil.rmtree(tmpdir)
-
-    def test_export_watchfile(self):
-        src = UScanSource(self.tree, False)
-        self.build_tree(['debian/', 'debian/watch'])
-        self.tree.smart_add(['debian/watch'])
-        tmpdir = tempfile.mkdtemp()
-        try:
-            self.assertIsNot(src._export_file('watch', tmpdir), None)
-        finally:
-            shutil.rmtree(tmpdir)
-
-    def test__xml_report_extract_upstream_version(self):
-        text = b"""
-<dehs>
-<package>tdb</package>
-<debian-uversion>1.2.8</debian-uversion>
-<debian-mangled-uversion>1.2.8</debian-mangled-uversion>
-<upstream-version>1.2.9</upstream-version>
-<upstream-url>ftp://ftp.samba.org/pub/tdb/tdb-1.2.9.tar.gz</upstream-url>
-<status>Newer version available</status>
-</dehs>"""
-        self.assertEquals("1.2.9",
-            UScanSource._xml_report_extract_upstream_version(text))
-
-    def test__xml_report_extract_upstream_version_warnings(self):
-        text = b"""
-<dehs>
-<package>tdb</package>
-<warnings>uscan warning: Unable to determine current version
-in debian/watch, skipping:
-ftp://ftp.samba.org/pub/tdb/tdb-(.+).tar.gz</warnings>
-</dehs>
-"""
-        self.assertIs(
-            None,
-            UScanSource._xml_report_extract_upstream_version(text))
-
-    def test__xml_report_extract_upstream_version_noise(self):
-        text = b"""
-<dehs>
-blahf =>
-<package>tdb</package>
-<debian-uversion>1.2.8</debian-uversion>
-<upstream-version>1.2.9</upstream-version>
-<warnings>uscan warning: Unable to determine current version
-in debian/watch, skipping:
-ftp://ftp.samba.org/pub/tdb/tdb-(.+).tar.gz</warnings>
-</dehs>
-"""
-        self.assertEquals("1.2.9",
-            UScanSource._xml_report_extract_upstream_version(text))
-
-    def test__xml_report_extract_upstream_version_errors(self):
-        text = b"""
-<dehs>
-blahf =>
-<package>tdb</package>
-<debian-uversion>1.2.8</debian-uversion>
-<errors>something something signature</errors>
-</dehs>
-"""
-        self.assertRaises(
-            UScanError,
-            UScanSource._xml_report_extract_upstream_version, text)
-
-
 class GuessUpstreamRevspecTests(TestCase):
 
     def test_guess_upstream_revspec(self):
@@ -503,6 +449,29 @@ class UpstreamBranchSourceTests(TestCaseWithTransport):
         self.tree.commit("msg")
         self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
 
+    def test_get_latest_version_not_snapshot(self):
+        self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
+        source = UpstreamBranchSource(self.tree.branch,
+            {"2.1": self.tree.branch.last_revision().decode('utf-8')},
+            snapshot=False)
+        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+        self.tree.commit("msg")
+        self.assertEquals("2.1", source.get_latest_version("foo", "1.0"))
+
+    def test_get_recent_versions(self):
+        self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
+        source = UpstreamBranchSource(
+            self.tree.branch,
+            {"2.1": self.tree.branch.last_revision().decode('utf-8')})
+        self.assertEquals(["2.1"], source.get_recent_versions("foo", "1.0"))
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag('2.2', revid2)
+        self.assertEquals(
+            ["2.1", "2.2"],
+            source.get_recent_versions("foo", "1.0"))
+
     def test_version_as_revisions(self):
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
@@ -616,6 +585,21 @@ class LazyUpstreamBranchSourceTests(TestCaseWithTransport):
         self.assertEquals("2.1+bzr2", source.get_latest_version("foo", "1.0"))
         self.assertIsNot(None, source._upstream_branch)
 
+    def test_get_recent_versions(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("2.1", revid1)
+        source = LazyUpstreamBranchSource(
+            self.tree.branch.base,
+            {"2.1": self.tree.branch.last_revision()})
+        self.assertIs(None, source._upstream_branch)
+        self.assertEquals(["2.1"], source.get_recent_versions("foo", "1.0"))
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("2.2", revid2)
+        self.assertEquals(
+            ["2.1", "2.2"],
+            source.get_recent_versions("foo", "1.0"))
+        self.assertIsNot(None, source._upstream_branch)
+
     def test_version_as_revision(self):
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("2.1", self.tree.branch.last_revision())
@@ -696,10 +680,18 @@ class TestUpstreamTagToVersion(TestCase):
         self.assertEquals("42.0", upstream_tag_to_version(u"bla-42.0", "bla"))
 
     def test_unicode(self):
-        self.assertEquals(
-                u"42.0\xa9",
+        self.assertIs(
+                None,
                 upstream_tag_to_version(u"bla-42.0\xa9", "bla"))
 
+    def test_popt(self):
+        self.assertEquals("1.4", upstream_tag_to_version("popt-1_4", "popt"))
+        self.assertEquals(
+            "1.4", upstream_tag_to_version("popt-1_4-release", "popt"))
+
+    def test_perl(self):
+        self.assertEquals("0.006019", upstream_tag_to_version("v0.006_019"))
+
 
 class TestUpstreamVersionAddRevision(TestCaseWithTransport):
     """Test that updating the version string works."""
@@ -775,6 +767,10 @@ class TestUpstreamVersionAddRevision(TestCaseWithTransport):
             "1.0~git20180101",
             upstream_version_add_revision(
                 self, "1.0~git20160320", b"somegitrev"))
+        self.assertEquals(
+            "1.0-git20180101",
+            upstream_version_add_revision(
+                self, "1.0-git20160320", b"somegitrev"))
         self.assertEquals(
             "1.0~git20180101.1.e7f47cf",
             upstream_version_add_revision(
@@ -917,12 +913,12 @@ class GenericPristineTarSourceTests(TestCase):
         self.assertEquals(b"bla", revision_pristine_tar_delta(rev))
 
 
-class PristineTarSourceTests(TestCaseWithTransport):
+class GitPristineTarSourceTests(TestCaseWithTransport):
 
     def setUp(self):
-        super(PristineTarSourceTests, self).setUp()
-        self.tree = self.make_branch_and_tree('unstable')
-        self.source = PristineTarSource(self.tree.branch)
+        super(GitPristineTarSourceTests, self).setUp()
+        self.tree = self.make_branch_and_tree('unstable', format='git')
+        self.source = GitPristineTarSource(self.tree.branch)
 
     def test_gbp_tag_format(self):
         self.build_tree_contents([
@@ -931,8 +927,7 @@ class PristineTarSourceTests(TestCaseWithTransport):
 [DEFAULT]
 upstream-tag = blah-%(version)s
 """)])
-        self.source = PristineTarSource.from_tree(
-            self.tree.branch, self.tree)
+        self.source = GitPristineTarSource.from_tree(self.tree)
         upstream_v_no = "0.1"
         self.assertEqual(
             self.source.tag_name(upstream_v_no),
@@ -945,12 +940,108 @@ upstream-tag = blah-%(version)s
 [DEFAULT]
 upstream-tag = blah-%(version%~%-)s
 """)])
-        self.source = PristineTarSource.from_tree(
-            self.tree.branch, self.tree)
+        self.source = GitPristineTarSource.from_tree(self.tree)
         self.assertEqual(
             self.source.tag_name("0.1~1"),
             "blah-0.1-1")
 
+    def test_upstream_tag_name(self):
+        upstream_v_no = "0.1"
+        self.assertEqual(
+            self.source.tag_name(upstream_v_no),
+            "upstream/" + upstream_v_no)
+
+    def test_tag_name_distro(self):
+        self.assertEquals(
+            self.source.tag_name("0.3", distro="ubuntu"),
+            "upstream-ubuntu/0.3")
+
+    def test_version(self):
+        self.assertEquals(
+            ['upstream/3.3', 'upstream-3.3', 'upstream_3.3', '3.3',
+             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3'],
+            self.source.possible_tag_names("pkg", "3.3", component=None))
+
+    def test_version_with_tilde(self):
+        self.assertEquals([
+            'upstream/3.3~brz232',
+            'upstream-3.3~brz232',
+            'upstream/3.3_brz232',
+            'upstream_3.3~brz232',
+            '3.3~brz232',
+            'v3.3~brz232',
+            'pkg-3.3~brz232'],
+            self.source.possible_tag_names(
+                'pkg', "3.3~brz232", component=None))
+
+    def test_version_component(self):
+        self.assertEquals([
+            'upstream-3.3/extlib'],
+            self.source.possible_tag_names('pkg', "3.3", component="extlib"))
+
+    def test_version_as_revisions_missing(self):
+        self.assertRaises(PackageVersionNotPresent,
+            self.source.version_as_revisions, None, "1.2")
+
+    def test_version_as_revisions_single(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.assertEquals({None: revid1},
+            self.source.version_as_revisions(None, "2.1"))
+
+    def test_version_component_as_revision(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid1)
+        self.assertEquals(revid1,
+            self.source.version_component_as_revision(None, "2.1", "lib"))
+
+    # git doesn't support subtags
+    @expectedFailure
+    def test_version_as_revisions(self):
+        revid1 = self.tree.commit("msg")
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid2)
+        self.assertEquals({None: revid1, "lib": revid2},
+                self.source.version_as_revisions(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
+
+    def test_version_as_revisions_partially_missing(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.assertRaises(PackageVersionNotPresent,
+                self.source.version_as_revisions, 'pkg', "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")])
+
+    @expectedFailure
+    def test_has_version_multiple(self):
+        revid1 = self.tree.commit("msg")
+        revid2 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.tree.branch.tags.set_tag("upstream-2.1/lib", revid2)
+        self.assertTrue(
+                self.source.has_version(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
+
+    def test_has_version_partially_missing(self):
+        revid1 = self.tree.commit("msg")
+        self.tree.branch.tags.set_tag("upstream-2.1", revid1)
+        self.assertFalse(
+                self.source.has_version(None, "2.1", [
+            ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
+            ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
+
+
+class BzrPristineTarSourceTests(TestCaseWithTransport):
+
+    def setUp(self):
+        super(BzrPristineTarSourceTests, self).setUp()
+        self.tree = self.make_branch_and_tree('unstable', format='bzr')
+        self.source = BzrPristineTarSource(self.tree.branch)
+
     def test_upstream_tag_name(self):
         upstream_v_no = "0.1"
         self.assertEqual(
@@ -963,30 +1054,27 @@ upstream-tag = blah-%(version%~%-)s
             "upstream-ubuntu-0.3")
 
     def test_version(self):
-        self.assertEquals([
-            'upstream-3.3',
-            'upstream-debian-3.3',
-            'upstream-ubuntu-3.3', 'upstream/3.3', 'upstream-3.3',
-            'upstream_3.3'],
-            self.source.possible_tag_names("3.3", component=None))
+        self.assertEquals(
+            ['upstream/3.3', 'upstream-3.3', 'upstream_3.3', '3.3',
+             'v3.3', 'release-3.3', 'v3.3-release', 'pkg-3.3'],
+            self.source.possible_tag_names("pkg", "3.3", component=None))
 
     def test_version_with_tilde(self):
         self.assertEquals([
-            'upstream-3.3~brz232',
-            'upstream-debian-3.3~brz232',
-            'upstream-ubuntu-3.3~brz232',
             'upstream/3.3~brz232',
             'upstream-3.3~brz232',
             'upstream/3.3_brz232',
-            'upstream_3.3~brz232'],
-            self.source.possible_tag_names("3.3~brz232", component=None))
+            'upstream_3.3~brz232',
+            '3.3~brz232',
+            'v3.3~brz232',
+            'pkg-3.3~brz232'],
+            self.source.possible_tag_names(
+                'pkg', "3.3~brz232", component=None))
 
     def test_version_component(self):
         self.assertEquals([
-            'upstream-3.3/extlib',
-            'upstream-debian-3.3/extlib',
-            'upstream-ubuntu-3.3/extlib'],
-            self.source.possible_tag_names("3.3", component="extlib"))
+            'upstream-3.3/extlib'],
+            self.source.possible_tag_names('pkg', "3.3", component="extlib"))
 
     def test_version_as_revisions_missing(self):
         self.assertRaises(PackageVersionNotPresent,
@@ -1018,7 +1106,7 @@ upstream-tag = blah-%(version%~%-)s
         revid1 = self.tree.commit("msg")
         self.tree.branch.tags.set_tag("upstream-2.1", revid1)
         self.assertRaises(PackageVersionNotPresent,
-                self.source.version_as_revisions, None, "2.1", [
+                self.source.version_as_revisions, 'pkg', "2.1", [
             ("upstream_2.1.orig.tar.gz", None, "somemd5sum"),
             ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")])
 
@@ -1041,6 +1129,8 @@ upstream-tag = blah-%(version%~%-)s
             ("upstream_2.1.orig-lib.tar.gz", "lib", "othermd5sum")]))
 
 
+
+
 class TarfileSourceTests(TestCaseWithTransport):
     """Tests for TarfileSource."""
 
@@ -1064,12 +1154,16 @@ class TarfileSourceTests(TestCaseWithTransport):
         source = TarfileSource(u"foo-1.0.tar.gz")
         latest_version = source.get_latest_version("foo", u"0.9")
         self.assertEquals("1.0", latest_version)
-        self.assertIsInstance(latest_version, text_type)
+        self.assertIsInstance(latest_version, str)
 
     def test_get_latest_version_parses(self):
         source = TarfileSource("foo-1.0.tar.gz")
         self.assertEquals("1.0", source.get_latest_version("foo", "0.9"))
 
+    def test_get_recent_versions(self):
+        source = TarfileSource("foo-1.0.tar.gz")
+        self.assertEquals(["1.0"], source.get_recent_versions("foo", "0.9"))
+
     def test_fetch_tarballs(self):
         source = TarfileSource("foo-1.0.tar.gz", "1.0")
         os.mkdir("bar")
