@@ -22,15 +22,13 @@ from __future__ import absolute_import
 
 import bz2
 import gzip
-try:
-    import hashlib as md5
-except ImportError:
-    import md5
+import hashlib
 import os
 import shutil
 import tarfile
 
 from debian.changelog import Changelog, Version
+from debmutate.changelog import strip_changelog_message
 
 from ..config import (
     BUILD_TYPE_MERGE,
@@ -58,9 +56,9 @@ from ..util import (
     find_changelog,
     find_extra_authors,
     find_last_distribution,
-    find_previous_upload,
     find_thanks,
     get_build_architecture,
+    get_files_excluded,
     get_commit_info_from_changelog,
     guess_build_type,
     lookup_distribution,
@@ -68,7 +66,6 @@ from ..util import (
     get_parent_dir,
     recursive_copy,
     safe_decode,
-    strip_changelog_message,
     suite_to_distribution,
     tarball_name,
     tree_contains_upstream_source,
@@ -78,7 +75,7 @@ from ..util import (
     )
 
 from .... import errors as bzr_errors
-from ....sixish import text_type
+from six import text_type
 from ....tests import (
     TestCase,
     )
@@ -110,6 +107,15 @@ class RecursiveCopyTests(TestCaseInTempDir):
         self.assertPathExists('a/d/e')
         self.assertPathExists('a/f')
 
+    def test_recursive_copy_symlink(self):
+        os.mkdir('a')
+        os.symlink('c', 'a/link')
+        os.mkdir('b')
+        recursive_copy('a', 'b')
+        self.assertPathExists('b')
+        self.assertPathExists('b/link')
+        self.assertEqual('c', os.readlink('b/link'))
+
 
 class SafeDecodeTests(TestCase):
 
@@ -158,7 +164,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         os.mkdir('debian')
         self.write_changelog('debian/changelog')
         tree.add(['debian', 'debian/changelog'])
-        (cl, lq) = find_changelog(tree, False)
+        (cl, lq) = find_changelog(tree, merge=False)
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, False)
 
@@ -167,7 +173,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         os.mkdir('debian')
         self.write_changelog('debian/changelog')
         tree.add(['debian', 'debian/changelog'])
-        (cl, lq) = find_changelog(tree, True)
+        (cl, lq) = find_changelog(tree, merge=True)
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, False)
 
@@ -175,7 +181,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
         tree.add(['changelog'])
-        (cl, lq) = find_changelog(tree, True)
+        (cl, lq) = find_changelog(tree, merge=True)
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, True)
 
@@ -187,22 +193,25 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         self.write_changelog('changelog')
         tree.add(['changelog'])
         os.symlink('.', 'debian')
-        self.assertRaises(AddChangelogError, find_changelog, tree, True)
+        self.assertRaises(AddChangelogError, find_changelog, tree, merge=True)
 
     def test_find_changelog_nomerge_lq(self):
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
         tree.add(['changelog'])
-        self.assertRaises(MissingChangelogError, find_changelog, tree, False)
+        self.assertRaises(
+            MissingChangelogError, find_changelog, tree, merge=False)
 
     def test_find_changelog_nochangelog(self):
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
-        self.assertRaises(MissingChangelogError, find_changelog, tree, False)
+        self.assertRaises(
+            MissingChangelogError, find_changelog, tree, merge=False)
 
     def test_find_changelog_nochangelog_merge(self):
         tree = self.make_branch_and_tree('.')
-        self.assertRaises(MissingChangelogError, find_changelog, tree, True)
+        self.assertRaises(
+            MissingChangelogError, find_changelog, tree, merge=True)
 
     def test_find_changelog_symlink(self):
         """When there was a symlink debian -> . then the code used to break"""
@@ -212,7 +221,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         tree.add(['changelog'])
         os.symlink('.', 'debian')
         tree.add(['debian'])
-        (cl, lq) = find_changelog(tree, True)
+        (cl, lq) = find_changelog(tree, merge=True)
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, True)
 
@@ -223,7 +232,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         with open('changelog', 'w') as f:
             f.write('Naughty, naughty')
         tree.add(['changelog', 'debian', 'debian/changelog'])
-        (cl, lq) = find_changelog(tree, True)
+        (cl, lq) = find_changelog(tree, merge=True)
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, False)
 
@@ -231,57 +240,27 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         tree = self.make_branch_and_tree('.')
         os.mkdir('debian')
         self.write_changelog('debian/changelog')
-        self.assertRaises(AddChangelogError, find_changelog, tree, False)
-
-
-class StripChangelogMessageTests(TestCase):
-
-    def test_None(self):
-        self.assertEqual(strip_changelog_message(None), None)
-
-    def test_no_changes(self):
-        self.assertEqual(strip_changelog_message([]), [])
-
-    def test_empty_changes(self):
-        self.assertEqual(strip_changelog_message(['']), [])
-
-    def test_removes_leading_whitespace(self):
-        self.assertEqual(strip_changelog_message(
-                    ['foo', '  bar', '\tbaz', '   bang']),
-                    ['foo', 'bar', 'baz', ' bang'])
-
-    def test_removes_star_if_one(self):
-        self.assertEqual(strip_changelog_message(['  * foo']), ['foo'])
-        self.assertEqual(strip_changelog_message(['\t* foo']), ['foo'])
-        self.assertEqual(strip_changelog_message(['  + foo']), ['foo'])
-        self.assertEqual(strip_changelog_message(['  - foo']), ['foo'])
-        self.assertEqual(strip_changelog_message(['  *  foo']), ['foo'])
-        self.assertEqual(strip_changelog_message(['  *  foo', '     bar']),
-                ['foo', 'bar'])
-
-    def test_leaves_start_if_multiple(self):
-        self.assertEqual(strip_changelog_message(['  * foo', '  * bar']),
-                    ['* foo', '* bar'])
-        self.assertEqual(strip_changelog_message(['  * foo', '  + bar']),
-                    ['* foo', '+ bar'])
-        self.assertEqual(strip_changelog_message(
-                    ['  * foo', '  bar', '  * baz']),
-                    ['* foo', 'bar', '* baz'])
+        self.assertRaises(AddChangelogError, find_changelog, tree, merge=False)
 
 
 class TarballNameTests(TestCase):
 
     def test_tarball_name(self):
-        self.assertEqual(tarball_name("package", "0.1", None),
-                "package_0.1.orig.tar.gz")
-        self.assertEqual(tarball_name("package", Version("0.1"), None),
-                "package_0.1.orig.tar.gz")
-        self.assertEqual(tarball_name("package", Version("0.1"), None,
-                    format='bz2'), "package_0.1.orig.tar.bz2")
-        self.assertEqual(tarball_name("package", Version("0.1"), None,
-                    format='xz'), "package_0.1.orig.tar.xz")
-        self.assertEqual(tarball_name("package", Version("0.1"), "la",
-                    format='xz'), "package_0.1.orig-la.tar.xz")
+        self.assertEqual(
+            tarball_name("package", "0.1", None),
+            "package_0.1.orig.tar.gz")
+        self.assertEqual(
+            tarball_name("package", Version("0.1"), None),
+            "package_0.1.orig.tar.gz")
+        self.assertEqual(
+            tarball_name("package", Version("0.1"), None, format='bz2'),
+            "package_0.1.orig.tar.bz2")
+        self.assertEqual(
+            tarball_name("package", Version("0.1"), None, format='xz'),
+            "package_0.1.orig.tar.xz")
+        self.assertEqual(
+            tarball_name("package", Version("0.1"), "la", format='xz'),
+            "package_0.1.orig-la.tar.xz")
 
 
 DistroInfoFeature = ModuleAvailableFeature('distro_info')
@@ -300,6 +279,9 @@ class SuiteToDistributionTests(TestCase):
     def lookup_debian(self, target):
         self.assertEqual(self._do_lookup(target), 'debian')
 
+    def lookup_kali(self, target):
+        self.assertEqual(self._do_lookup(target), 'kali')
+
     def lookup_other(self, target):
         self.assertEqual(self._do_lookup(target), None)
 
@@ -316,6 +298,11 @@ class SuiteToDistributionTests(TestCase):
         self.lookup_debian('testing-proposed-updates')
         self.lookup_debian('etch-backports')
 
+    def test_lookup_kali(self):
+        self.lookup_kali('kali-dev')
+        self.lookup_kali('kali-rolling')
+        self.lookup_kali('kali')
+
     def test_lookup_other(self):
         self.lookup_other('not-a-target')
         self.lookup_other("debian")
@@ -351,7 +338,7 @@ class MoveFileTests(TestCaseInTempDir):
 
     def test_move_file_same_md5(self):
         self.build_tree(['a'])
-        md5sum = md5.md5()
+        md5sum = hashlib.md5()
         with open('a', 'rb') as f:
             md5sum.update(f.read())
         shutil.copy('a', 'b')
@@ -361,11 +348,11 @@ class MoveFileTests(TestCaseInTempDir):
 
     def test_move_file_diff_md5(self):
         self.build_tree(['a', 'b'])
-        md5sum = md5.md5()
+        md5sum = hashlib.md5()
         with open('a', 'rb') as f:
             md5sum.update(f.read())
         a_hexdigest = md5sum.hexdigest()
-        md5sum = md5.md5()
+        md5sum = hashlib.md5()
         with open('b', 'rb') as f:
             md5sum.update(f.read())
         b_hexdigest = md5sum.hexdigest()
@@ -373,7 +360,7 @@ class MoveFileTests(TestCaseInTempDir):
         move_file_if_different('a', 'b', a_hexdigest)
         self.assertPathDoesNotExist('a')
         self.assertPathExists('b')
-        md5sum = md5.md5()
+        md5sum = hashlib.md5()
         with open('b', 'rb') as f:
             md5sum.update(f.read())
         self.assertEqual(md5sum.hexdigest(), a_hexdigest)
@@ -433,8 +420,9 @@ class DgetTests(TestCaseWithTransport):
         builder.add_default_control()
         # No builder.build()
         self.build_tree(["target/"])
-        self.assertRaises(bzr_errors.NoSuchFile, dget,
-                self.get_url(builder.dsc_name()), 'target')
+        self.assertRaises(
+            bzr_errors.NoSuchFile, dget,
+            self.get_url(builder.dsc_name()), 'target')
 
     def test_dget_missing_file(self):
         builder = SourcePackageBuilder("package", Version("0.1-1"))
@@ -443,16 +431,18 @@ class DgetTests(TestCaseWithTransport):
         builder.build()
         os.unlink(builder.tar_name())
         self.build_tree(["target/"])
-        self.assertRaises(bzr_errors.NoSuchFile, dget,
-                self.get_url(builder.dsc_name()), 'target')
+        self.assertRaises(
+            bzr_errors.NoSuchFile, dget,
+            self.get_url(builder.dsc_name()), 'target')
 
     def test_dget_missing_target(self):
         builder = SourcePackageBuilder("package", Version("0.1-1"))
         builder.add_upstream_file("foo")
         builder.add_default_control()
         builder.build()
-        self.assertRaises(bzr_errors.NotADirectory, dget,
-                self.get_url(builder.dsc_name()), 'target')
+        self.assertRaises(
+            bzr_errors.NotADirectory, dget,
+            self.get_url(builder.dsc_name()), 'target')
 
     def test_dget_changes(self):
         builder = SourcePackageBuilder("package", Version("0.1-1"))
@@ -559,49 +549,54 @@ class ChangelogInfoTests(TestCaseWithTransport):
         wt = self.make_branch_and_tree(".")
         changes = ["  * Closes: #12345, 56789", "  * closes:bug45678"]
         bugs = find_bugs_fixed(changes, wt.branch, _lplib=MockLaunchpad())
-        self.assertEqual(["http://bugs.debian.org/12345 fixed",
-                "http://bugs.debian.org/56789 fixed",
-                "http://bugs.debian.org/45678 fixed"], bugs)
+        self.assertEqual(
+            ["http://bugs.debian.org/12345 fixed",
+             "http://bugs.debian.org/56789 fixed",
+             "http://bugs.debian.org/45678 fixed"], bugs)
 
     def test_find_bugs_fixed_debian_with_ubuntu_links(self):
         wt = self.make_branch_and_tree(".")
         changes = ["  * Closes: #12345", "  * closes:bug45678"]
-        lplib = MockLaunchpad(debian_bug_to_ubuntu_bugs=
-                {"12345": ("998877", "987654"),
+        lplib = MockLaunchpad(
+            debian_bug_to_ubuntu_bugs={
+                "12345": ("998877", "987654"),
                 "45678": ("87654",)})
         bugs = find_bugs_fixed(changes, wt.branch, _lplib=lplib)
         self.assertEqual([], lplib.ubuntu_bug_lookups)
         self.assertEqual(["12345", "45678"], lplib.debian_bug_lookups)
-        self.assertEqual(["http://bugs.debian.org/12345 fixed",
-                "http://bugs.debian.org/45678 fixed",
-                "https://launchpad.net/bugs/87654 fixed"], bugs)
+        self.assertEqual(
+            ["http://bugs.debian.org/12345 fixed",
+             "http://bugs.debian.org/45678 fixed",
+             "https://launchpad.net/bugs/87654 fixed"], bugs)
 
     def test_find_bugs_fixed_lp(self):
         wt = self.make_branch_and_tree(".")
         changes = ["  * LP: #12345,#56789", "  * lp:  #45678"]
         bugs = find_bugs_fixed(changes, wt.branch, _lplib=MockLaunchpad())
-        self.assertEqual(["https://launchpad.net/bugs/12345 fixed",
-                "https://launchpad.net/bugs/56789 fixed",
-                "https://launchpad.net/bugs/45678 fixed"], bugs)
+        self.assertEqual(
+            ["https://launchpad.net/bugs/12345 fixed",
+             "https://launchpad.net/bugs/56789 fixed",
+             "https://launchpad.net/bugs/45678 fixed"], bugs)
 
     def test_find_bugs_fixed_lp_with_debian_links(self):
         wt = self.make_branch_and_tree(".")
         changes = ["  * LP: #12345", "  * lp:  #45678"]
-        lplib = MockLaunchpad(ubuntu_bug_to_debian_bugs=
-                {"12345": ("998877", "987654"), "45678": ("87654",)})
+        lplib = MockLaunchpad(
+            ubuntu_bug_to_debian_bugs={
+                "12345": ("998877", "987654"), "45678": ("87654",)})
         bugs = find_bugs_fixed(changes, wt.branch, _lplib=lplib)
         self.assertEqual([], lplib.debian_bug_lookups)
         self.assertEqual(["12345", "45678"], lplib.ubuntu_bug_lookups)
-        self.assertEqual(["https://launchpad.net/bugs/12345 fixed",
-                "https://launchpad.net/bugs/45678 fixed",
-                "http://bugs.debian.org/87654 fixed"], bugs)
+        self.assertEqual(
+            ["https://launchpad.net/bugs/12345 fixed",
+             "https://launchpad.net/bugs/45678 fixed",
+             "http://bugs.debian.org/87654 fixed"], bugs)
 
     def test_get_commit_info_none(self):
         wt = self.make_branch_and_tree(".")
         changelog = Changelog()
-        message, authors, thanks, bugs = \
-                get_commit_info_from_changelog(changelog, wt.branch,
-                        _lplib=MockLaunchpad())
+        message, authors, thanks, bugs = get_commit_info_from_changelog(
+            changelog, wt.branch, _lplib=MockLaunchpad())
         self.assertEqual(None, message)
         self.assertEqual([], authors)
         self.assertEqual([], thanks)
@@ -614,24 +609,22 @@ class ChangelogInfoTests(TestCaseWithTransport):
                    "  * Second change, thanks to B. Hacker"]
         author = "J. Maintainer <maint@example.com"
         changelog.new_block(changes=changes, author=author)
-        message, authors, thanks, bugs = \
-                get_commit_info_from_changelog(changelog, wt.branch,
-                        _lplib=MockLaunchpad())
+        message, authors, thanks, bugs = get_commit_info_from_changelog(
+            changelog, wt.branch, _lplib=MockLaunchpad())
         self.assertEqual("\n".join(strip_changelog_message(changes)), message)
         self.assertEqual([author]+find_extra_authors(changes), authors)
         self.assertEqual(text_type, type(authors[0]))
         self.assertEqual(find_thanks(changes), thanks)
-        self.assertEqual(find_bugs_fixed(changes, wt.branch,
-                    _lplib=MockLaunchpad()), bugs)
+        self.assertEqual(find_bugs_fixed(
+            changes, wt.branch, _lplib=MockLaunchpad()), bugs)
 
     def assertUnicodeCommitInfo(self, changes):
         wt = self.make_branch_and_tree(".")
         changelog = Changelog()
         author = "J. Maintainer <maint@example.com>"
         changelog.new_block(changes=changes, author=author)
-        message, authors, thanks, bugs = \
-                get_commit_info_from_changelog(changelog, wt.branch,
-                        _lplib=MockLaunchpad())
+        message, authors, thanks, bugs = get_commit_info_from_changelog(
+            changelog, wt.branch, _lplib=MockLaunchpad())
         self.assertEqual(u'[ \xc1. Hacker ]\n'
                          u'* First ch\xe1nge, LP: #12345\n'
                          u'* Second change, thanks to \xde. Hacker',
@@ -651,7 +644,7 @@ class ChangelogInfoTests(TestCaseWithTransport):
 class MockLaunchpad(object):
 
     def __init__(self, debian_bug_to_ubuntu_bugs={},
-            ubuntu_bug_to_debian_bugs={}):
+                 ubuntu_bug_to_debian_bugs={}):
         self.debian_bug_to_ubuntu_bugs = debian_bug_to_ubuntu_bugs
         self.ubuntu_bug_to_debian_bugs = ubuntu_bug_to_debian_bugs
         self.debian_bug_lookups = []
@@ -709,46 +702,55 @@ class FindPreviousUploadTests(TestCase):
         author = "J. Maintainer <maint@example.com>"
         for version, distro in versions_and_distributions:
             cl.new_block(changes=changes, author=author,
-                                distributions=distro, version=version)
+                         distributions=distro, version=version)
         return cl
 
     def test_find_previous_upload_debian(self):
-        cl = self.make_changelog([("0.1-1", "unstable"),
-                ("0.1-2", "unstable")])
+        cl = self.make_changelog(
+            [("0.1-1", "unstable"),
+             ("0.1-2", "unstable")])
         self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
-        cl = self.make_changelog([("0.1-1", "unstable"),
-                ("0.1-1.1", "stable-security"), ("0.1-2", "unstable")])
+        cl = self.make_changelog(
+            [("0.1-1", "unstable"),
+             ("0.1-1.1", "stable-security"), ("0.1-2", "unstable")])
         self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_ubuntu(self):
-        cl = self.make_changelog([("0.1-1", "lucid"),
-                ("0.1-2", "lucid")])
+        cl = self.make_changelog(
+            [("0.1-1", "lucid"),
+             ("0.1-2", "lucid")])
         self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
-        cl = self.make_changelog([("0.1-1", "lucid"),
-                ("0.1-1.1", "unstable"), ("0.1-2", "maverick")])
+        cl = self.make_changelog(
+            [("0.1-1", "lucid"),
+             ("0.1-1.1", "unstable"), ("0.1-2", "maverick")])
         self.requireFeature(DistroInfoFeature)
-        self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
+        self.assertEqual(
+                Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_ubuntu_pocket(self):
-        cl = self.make_changelog([("0.1-1", "lucid-updates"),
-                ("0.1-2", "lucid-updates")])
+        cl = self.make_changelog(
+            [("0.1-1", "lucid-updates"),
+             ("0.1-2", "lucid-updates")])
         self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
     def test_find_previous_upload_unknown(self):
-        cl = self.make_changelog([("0.1-1", "lucid"),
-                ("0.1-2", "dunno")])
+        cl = self.make_changelog(
+            [("0.1-1", "lucid"),
+             ("0.1-2", "dunno")])
         self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
 
     def test_find_previous_upload_missing(self):
-        cl = self.make_changelog([("0.1-1", "unstable"),
-                ("0.1-2", "lucid")])
+        cl = self.make_changelog(
+            [("0.1-1", "unstable"),
+             ("0.1-2", "lucid")])
         self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
         cl = self.make_changelog([("0.1-1", "unstable")])
         self.assertRaises(NoPreviousUpload, changelog_find_previous_upload, cl)
 
     def test_find_previous_upload_unreleased(self):
-        cl = self.make_changelog([("0.1-1", "unstable"),
-                ("0.1-2", "UNRELEASED")])
+        cl = self.make_changelog(
+            [("0.1-1", "unstable"),
+             ("0.1-2", "UNRELEASED")])
         self.assertEqual(Version("0.1-1"), changelog_find_previous_upload(cl))
 
 
@@ -760,22 +762,25 @@ class SourceFormatTests(TestCaseWithTransport):
 
     def test_source_format_newline(self):
         tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([("debian/", ), ("debian/source/",),
-            ("debian/source/format", "3.0 (native)\n")])
+        self.build_tree_contents(
+            [("debian/", ), ("debian/source/",),
+             ("debian/source/format", "3.0 (native)\n")])
         tree.add(["debian", "debian/source", "debian/source/format"])
         self.assertEquals("3.0 (native)", tree_get_source_format(tree))
 
     def test_source_format(self):
         tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([("debian/",), ("debian/source/",),
-            ("debian/source/format", "3.0 (quilt)")])
+        self.build_tree_contents(
+            [("debian/",), ("debian/source/",),
+             ("debian/source/format", "3.0 (quilt)")])
         tree.add(["debian", "debian/source", "debian/source/format"])
         self.assertEquals("3.0 (quilt)", tree_get_source_format(tree))
 
     def test_source_format_file_unversioned(self):
         tree = self.make_branch_and_tree('.')
-        self.build_tree_contents([("debian/",), ("debian/source/",),
-            ("debian/source/format", "3.0 (quilt)")])
+        self.build_tree_contents(
+            [("debian/",), ("debian/source/",),
+             ("debian/source/format", "3.0 (quilt)")])
         self.assertEquals("3.0 (quilt)", tree_get_source_format(tree))
 
 
@@ -788,63 +793,79 @@ class GuessBuildTypeTests(TestCaseWithTransport):
         :param tree: Tree to write to.
         :param format_string: Format string to write.
         """
-        self.build_tree_contents([("debian/",), ("debian/source/",),
-            ("debian/source/format", format_string)])
+        self.build_tree_contents(
+            [("debian/",), ("debian/source/",),
+             ("debian/source/format", format_string)])
         tree.add(["debian", "debian/source", "debian/source/format"])
 
     def test_normal_source_format(self):
         # Normal source format -> NORMAL
         tree = self.make_branch_and_tree('.')
         self.writeVersionFile(tree, "3.0 (quilt)")
-        self.assertEquals(BUILD_TYPE_NORMAL, guess_build_type(tree, None, True))
+        self.assertEquals(
+            BUILD_TYPE_NORMAL,
+            guess_build_type(tree, None, contains_upstream_source=True))
 
     def test_normal_source_format_merge(self):
         # Normal source format without upstream source -> MERGE
         tree = self.make_branch_and_tree('.')
         self.writeVersionFile(tree, "3.0 (quilt)")
-        self.assertEquals(BUILD_TYPE_MERGE, guess_build_type(tree, None, False))
+        self.assertEquals(
+            BUILD_TYPE_MERGE,
+            guess_build_type(tree, None, contains_upstream_source=False))
 
     def test_native_source_format(self):
         # Native source format -> NATIVE
         tree = self.make_branch_and_tree('.')
         self.writeVersionFile(tree, "3.0 (native)")
-        self.assertEquals(BUILD_TYPE_NATIVE, guess_build_type(tree, None, True))
+        self.assertEquals(
+            BUILD_TYPE_NATIVE,
+            guess_build_type(tree, None, contains_upstream_source=True))
 
     def test_prev_version_native(self):
         # Native package version -> NATIVE
         tree = self.make_branch_and_tree('.')
-        self.assertEquals(BUILD_TYPE_NATIVE,
-            guess_build_type(tree, Version("1.0"), True))
+        self.assertEquals(
+            BUILD_TYPE_NATIVE,
+            guess_build_type(
+                tree, Version("1.0"), contains_upstream_source=True))
 
     def test_empty(self):
         # Empty tree and a non-native package -> NORMAL
         tree = self.make_branch_and_tree('.')
-        self.assertEquals(BUILD_TYPE_NORMAL,
-            guess_build_type(tree, Version("1.0-1"), None))
+        self.assertEquals(
+            BUILD_TYPE_NORMAL,
+            guess_build_type(
+                tree, Version("1.0-1"), contains_upstream_source=None))
 
     def test_no_upstream_source(self):
         # No upstream source code and a non-native package -> MERGE
         tree = self.make_branch_and_tree('.')
         tree.mkdir("debian")
-        self.assertEquals(BUILD_TYPE_MERGE,
-            guess_build_type(tree, Version("1.0-1"), False))
+        self.assertEquals(
+            BUILD_TYPE_MERGE,
+            guess_build_type(
+                tree, Version("1.0-1"), contains_upstream_source=False))
 
     def test_default(self):
         # Upstream source code and a non-native package -> NORMAL
         tree = self.make_branch_and_tree('.')
-        self.assertEquals(BUILD_TYPE_NORMAL,
-            guess_build_type(tree, Version("1.0-1"), True))
+        self.assertEquals(
+            BUILD_TYPE_NORMAL,
+            guess_build_type(
+                tree, Version("1.0-1"), contains_upstream_source=True))
 
     def test_inconsistent(self):
-        # If version string and source format disagree on whether the package is native,
-        # raise an exception.
+        # If version string and source format disagree on whether the package
+        # is native, raise an exception.
         tree = self.make_branch_and_tree('.')
-        self.writeVersionFile(tree, "3.0 (native)")
-        e = self.assertRaises(InconsistentSourceFormatError, guess_build_type, tree,
-            Version("1.0-1"), True)
+        self.writeVersionFile(tree, "3.0 (quilt)")
+        e = self.assertRaises(
+            InconsistentSourceFormatError, guess_build_type, tree,
+            Version("1.0"), contains_upstream_source=True)
         self.assertEquals(
-            "Inconsistency between source format and version: version is not native, "
-            "format is native.", str(e))
+            "Inconsistency between source format and version: "
+            "version is native, format is not native.", str(e))
 
 
 class TestExtractOrigTarballs(TestCaseInTempDir):
@@ -870,8 +891,8 @@ class TestExtractOrigTarballs(TestCaseInTempDir):
                 import lzma
                 f = lzma.LZMAFile(tar_path, "w")
             else:
-                raise AssertionError("Unknown compressin type %r" %
-                    compression)
+                raise AssertionError(
+                    "Unknown compressin type %r" % compression)
             try:
                 tf = tarfile.open(None, 'w', f)
                 try:
@@ -887,55 +908,67 @@ class TestExtractOrigTarballs(TestCaseInTempDir):
     def test_single_orig_tar_gz(self):
         tar_path = self.create_tarball("package", "0.1", "gz")
         os.mkdir("target")
-        extract_orig_tarballs([(tar_path, None)], "target",
-            strip_components=1)
+        extract_orig_tarballs(
+            [(tar_path, None)], "target", strip_components=1)
         self.assertEquals(os.listdir("target"), ["README"])
 
     def test_single_orig_tar_bz2(self):
         tar_path = self.create_tarball("package", "0.1", "bz2")
         os.mkdir("target")
-        extract_orig_tarballs([(tar_path, None)], "target",
-            strip_components=1)
+        extract_orig_tarballs(
+            [(tar_path, None)], "target", strip_components=1)
         self.assertEquals(os.listdir("target"), ["README"])
 
     def test_single_orig_tar_xz(self):
         self.requireFeature(LzmaFeature)
         tar_path = self.create_tarball("package", "0.1", "xz")
         os.mkdir("target")
-        extract_orig_tarballs([(tar_path, None)], "target",
-            strip_components=1)
+        extract_orig_tarballs(
+            [(tar_path, None)], "target", strip_components=1)
         self.assertEquals(os.listdir("target"), ["README"])
 
     def test_multiple_tarballs(self):
         base_tar_path = self.create_tarball("package", "0.1", "bz2")
-        tar_path_extra = self.create_tarball("package", "0.1", "bz2", part="extra")
+        tar_path_extra = self.create_tarball(
+            "package", "0.1", "bz2", part="extra")
         os.mkdir("target")
-        extract_orig_tarballs([(base_tar_path, None), (tar_path_extra, "extra")], "target",
+        extract_orig_tarballs(
+            [(base_tar_path, None), (tar_path_extra, "extra")], "target",
             strip_components=1)
-        self.assertEquals(sorted(os.listdir("target")),
+        self.assertEquals(
+            sorted(os.listdir("target")),
             sorted(["README", "extra"]))
 
 
 class ComponentFromOrigTarballTests(TestCase):
 
     def test_base_tarball(self):
-        self.assertIs(None,
-            component_from_orig_tarball("foo_0.1.orig.tar.gz", "foo", "0.1"))
-        self.assertRaises(ValueError,
+        self.assertIs(
+            None,
+            component_from_orig_tarball(
+                "foo_0.1.orig.tar.gz", "foo", "0.1"))
+        self.assertRaises(
+            ValueError,
             component_from_orig_tarball, "foo_0.1.orig.tar.gz", "bar", "0.1")
 
     def test_invalid_extension(self):
-        self.assertRaises(ValueError,
+        self.assertRaises(
+            ValueError,
             component_from_orig_tarball, "foo_0.1.orig.unknown", "foo", "0.1")
 
     def test_component(self):
-        self.assertEquals("comp",
-            component_from_orig_tarball("foo_0.1.orig-comp.tar.gz", "foo", "0.1"))
-        self.assertEquals("comp-dash",
-            component_from_orig_tarball("foo_0.1.orig-comp-dash.tar.gz", "foo", "0.1"))
+        self.assertEquals(
+            "comp",
+            component_from_orig_tarball(
+                "foo_0.1.orig-comp.tar.gz", "foo", "0.1"))
+        self.assertEquals(
+            "comp-dash",
+            component_from_orig_tarball(
+                "foo_0.1.orig-comp-dash.tar.gz", "foo", "0.1"))
 
     def test_invalid_character(self):
-        self.assertRaises(ValueError,
+        self.assertRaises(
+            ValueError,
             component_from_orig_tarball, "foo_0.1.orig;.tar.gz", "foo", "0.1")
 
 
@@ -984,3 +1017,43 @@ class BuildArchitectureTests(TestCase):
 
     def test_is_str(self):
         self.assertIsInstance(get_build_architecture(), text_type)
+
+
+class FilesExcludedTests(TestCaseWithTransport):
+
+    def test_file_missing(self):
+        tree = self.make_branch_and_tree('.')
+        self.assertRaises(bzr_errors.NoSuchFile, get_files_excluded, tree)
+
+    def test_not_set(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            ('debian/', ),
+            ('debian/copyright', """\
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: blah
+
+Files: *
+Copyright:
+ (c) Somebody
+License: MIT
+""")])
+        tree.add(['debian', 'debian/copyright'])
+        self.assertEqual([], get_files_excluded(tree))
+
+    def test_set(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            ('debian/', ),
+            ('debian/copyright', """\
+Format: https://www.debian.org/doc/packaging-manuals/copyright-format/1.0/
+Upstream-Name: blah
+Files-Excluded: blah/* flattr.png
+
+Files: *
+Copyright:
+ (c) Somebody
+License: MIT
+""")])
+        tree.add(['debian', 'debian/copyright'])
+        self.assertEqual(['blah/*', 'flattr.png'], get_files_excluded(tree))
