@@ -28,19 +28,14 @@ import tempfile
 
 from debian.changelog import Version
 
-from ....errors import BzrError, NoSuchFile
+from ....errors import BzrError, DependencyNotPresent, NoSuchFile
 from .... import osutils
 from ....export import export
-from ....sixish import PY3
 from ....trace import (
     note,
     warning,
     )
 
-from ..errors import (
-    MissingUpstreamTarball,
-    PackageVersionNotPresent,
-    )
 from ..repack_tarball import (
     get_filetype,
     repack_tarball,
@@ -51,6 +46,22 @@ from ..util import (
     )
 
 
+class PackageVersionNotPresent(BzrError):
+    _fmt = "%(package)s %(version)s was not found in %(upstream)s."
+
+    def __init__(self, package, version, upstream):
+        BzrError.__init__(self, package=package, version=version,
+                          upstream=upstream)
+
+
+class MissingUpstreamTarball(BzrError):
+    _fmt = ("Unable to find the needed upstream tarball for package "
+            "%(package)s, version %(version)s.")
+
+    def __init__(self, package, version):
+        BzrError.__init__(self, package=package, version=version)
+
+
 def new_tarball_name(package, version, old_name):
     """Determine new tarball name based on the package name and the old name.
     """
@@ -127,7 +138,10 @@ class AptSource(UpstreamSource):
     def fetch_tarballs(self, package, upstream_version, target_dir,
                        _apt_pkg=None, components=None):
         if _apt_pkg is None:
-            import apt_pkg
+            try:
+                import apt_pkg
+            except ImportError as e:
+                raise DependencyNotPresent('apt_pkg', e)
         else:
             apt_pkg = _apt_pkg
         apt_pkg.init()
@@ -173,152 +187,20 @@ class AptSource(UpstreamSource):
         return True
 
 
-class UScanError(BzrError):
-
-    _fmt = "UScan failed to run: %(errors)s."
-
-    def __init__(self, errors):
-        self.errors = errors
-
-
-class UScanSource(UpstreamSource):
-    """Upstream source that uses uscan."""
-
-    def __init__(self, tree, subpath=None, top_level=False):
-        self.tree = tree
-        self.subpath = subpath
-        self.top_level = top_level
-
-    def _export_file(self, name, directory):
-        if self.top_level:
-            file = name
-        else:
-            file = 'debian/' + name
-        if self.subpath:
-            file = osutils.pathjoin(self.subpath, file)
-        if not self.tree.has_filename(file):
-            raise NoSuchFile(file, self.tree)
-        output_path = os.path.join(directory, 'debian', name)
-        output_dir = os.path.dirname(output_path)
-        if not os.path.isdir(output_dir):
-            os.makedirs(output_dir)
-        with open(output_path, 'wb') as f:
-            f.write(self.tree.get_file_text(file))
-        return output_path
-
-    @staticmethod
-    def _run_dehs_uscan(args, cwd):
-        p = subprocess.Popen(
-            ["uscan", "--dehs"] + args,
-            stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
-        (stdout, stderr) = p.communicate()
-        if b'</dehs>' not in stdout:
-            error = stderr.decode()
-            if error.startswith('uscan error '):
-                error = error[len('uscan error '):]
-            raise UScanError(error)
-        if PY3:
-            sys.stderr.write(stderr.decode())
-        else:
-            sys.stderr.write(stderr)
-        return stdout, p.returncode
-
-    @staticmethod
-    def _xml_report_extract_warnings(text):
-        from xml.sax.saxutils import unescape
-        for m in re.finditer(b"<warnings>(.*)</warnings>", text):
-            warning(unescape(m.group(1).decode()))
-
-    @staticmethod
-    def _xml_report_extract_errors(text):
-        from xml.sax.saxutils import unescape
-        for m in re.finditer(b"<errors>(.*)</errors>", text):
-            raise UScanError(unescape(m.group(1).decode()))
-
-    @staticmethod
-    def _xml_report_extract_upstream_version(text):
-        UScanSource._xml_report_extract_warnings(text)
-        UScanSource._xml_report_extract_errors(text)
-        from xml.sax.saxutils import unescape
-        # uscan --dehs's output isn't well-formed XML, so let's fall back to
-        # regexes instead..
-        m = re.search(b'<upstream-version>(.*)</upstream-version>', text)
-        if not m:
-            return None
-        return unescape(m.group(1).decode())
-
-    @staticmethod
-    def _xml_report_extract_target_paths(text):
-        UScanSource._xml_report_extract_warnings(text)
-        UScanSource._xml_report_extract_errors(text)
-        from xml.sax.saxutils import unescape
-        return [
-            unescape(m.group(1).decode())
-            for m in re.finditer(b'<target-path>(.*)</target-path>', text)]
-
-    def get_latest_version(self, package, current_version):
-        tmpdir = tempfile.mkdtemp()
-        try:
-            try:
-                watch_tempfilename = self._export_file('watch', tmpdir)
-            except NoSuchFile:
-                note("No watch file to use to check latest upstream release.")
-                return None
-            args = ["--watchfile=%s" % watch_tempfilename,
-                    "--package=%s" % package, "--report",
-                    "--no-download", "--upstream-version=%s" % current_version]
-            text, retcode = self._run_dehs_uscan(args, cwd=tmpdir)
-        finally:
-            shutil.rmtree(tmpdir)
-        return self._xml_report_extract_upstream_version(text)
-
-    def get_recent_versions(self, package, since_version=None):
-        raise NotImplementedError(self.get_recent_versions)
-
-    def fetch_tarballs(self, package, version, target_dir, components=None):
-        note("Using uscan to look for the upstream tarball.")
-        tmpdir = tempfile.mkdtemp()
-        try:
-            # Just export all of debian/, since e.g. uupdate needs more of it.
-            export(self.tree, os.path.join(tmpdir, 'debian'), format='dir',
-                   subdir='debian')
-            args = ["--force-download", "--rename",
-                    "--check-dirname-level=0",
-                    "--download", "--destdir=%s" % target_dir,
-                    "--download-version=%s" % version]
-            text, r = self._run_dehs_uscan(args, cwd=tmpdir)
-            orig_files = self._xml_report_extract_target_paths(text)
-            if not orig_files:
-                note("uscan could not find the needed tarball.")
-                raise PackageVersionNotPresent(package, version, self)
-        finally:
-            shutil.rmtree(tmpdir)
-        if not orig_files:
-            note("the expected files generated by uscan could not be found in"
-                 "%s.", target_dir)
-            raise PackageVersionNotPresent(package, version, self)
-        return orig_files
-
-
 class SelfSplitSource(UpstreamSource):
 
     def __init__(self, tree):
         self.tree = tree
 
     def _split(self, package, upstream_version, target_filename):
-        tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
-        try:
+        with tempfile.TemporaryDirectory(
+                prefix="builddeb-get-orig-source-") as tmpdir:
             export_dir = os.path.join(
                 tmpdir, "%s-%s" % (package, upstream_version))
             export(self.tree, export_dir, format="dir")
             shutil.rmtree(os.path.join(export_dir, "debian"))
-            tar = tarfile.open(target_filename, "w:gz")
-            try:
+            with tarfile.open(target_filename, "w:gz") as tar:
                 tar.add(export_dir, "%s-%s" % (package, upstream_version))
-            finally:
-                tar.close()
-        finally:
-            shutil.rmtree(tmpdir)
 
     def fetch_tarballs(self, package, version, target_dir, components=None):
         note("Using the current branch without the 'debian' directory "
@@ -347,6 +229,9 @@ class StackedUpstreamSource(UpstreamSource):
                     package, version, target_dir, components)
             except PackageVersionNotPresent:
                 pass
+            except DependencyNotPresent as e:
+                warning('not checking %r due to missing dependency: %s',
+                        source, e)
             else:
                 return paths
         raise PackageVersionNotPresent(package, version, self)
@@ -374,6 +259,9 @@ class StackedUpstreamSource(UpstreamSource):
                 return source.version_as_revisions(package, version, tarballs)
             except PackageVersionNotPresent:
                 pass
+            except DependencyNotPresent as e:
+                warning('not checking %r due to missing dependency: %s',
+                        source, e)
         raise PackageVersionNotPresent(package, version, self)
 
     def has_version(self, package, version, tarballs=None):
@@ -454,8 +342,9 @@ class UpstreamProvider(object):
         if in_target is not None:
             note("Upstream tarball already exists in build directory, "
                  "using that")
-            return [(p, component_from_orig_tarball(p,
-                self.package, self.version)) for p in in_target]
+            return [
+                (p, component_from_orig_tarball(p, self.package, self.version))
+                for p in in_target]
         if self.already_exists_in_store() is None:
             if not os.path.exists(self.store_dir):
                 os.makedirs(self.store_dir)
@@ -484,8 +373,8 @@ class UpstreamProvider(object):
         if paths is None:
             return None
         for path in paths:
-            repack_tarball(path, os.path.basename(path),
-                    target_dir=target_dir)
+            repack_tarball(
+                path, os.path.basename(path), target_dir=target_dir)
         return paths
 
 
@@ -497,7 +386,7 @@ def extract_tarball_version(path, packagename):
     """
     basename = os.path.basename(path)
     for extension in [".tar.gz", ".tgz", ".tar.bz2", ".tar.lzma", ".tar.xz",
-            ".zip"]:
+                      ".zip"]:
         if basename.endswith(extension):
             basename = basename[:-len(extension)]
             break
@@ -592,8 +481,8 @@ class LaunchpadReleaseFileSource(UpstreamSource):
             warning("More than one release file for release %s of package %s"
                     "found on Launchpad. Using the first.", version, package)
         hosted_file = release_files[0]
-        tmpdir = tempfile.mkdtemp(prefix="builddeb-launchpad-source-")
-        try:
+        with tempfile.TemporaryDirectory(
+                prefix="builddeb-launchpad-source-") as tmpdir:
             with hosted_file.open() as inf:
                 note("Downloading upstream tarball %s from Launchpad",
                      inf.filename)
@@ -605,8 +494,6 @@ class LaunchpadReleaseFileSource(UpstreamSource):
             dest_name = new_tarball_name(package, version, filename)
             repack_tarball(tmppath, dest_name, target_dir=target_dir)
             return os.path.join(target_dir, dest_name)
-        finally:
-            shutil.rmtree(tmpdir)
 
     def _all_versions(self):
         for release in self.project_series.releases:
@@ -623,3 +510,31 @@ class LaunchpadReleaseFileSource(UpstreamSource):
         versions = list(self._all_versions())
         versions.sort()
         return versions[-1][1]
+
+
+class DirectoryScanSource(UpstreamSource):
+    """Source that scans a local directory for sources."""
+
+    def __init__(self, path):
+        self.path = os.path.abspath(path)
+
+    def fetch_tarballs(self, package, version, target_dir, components=None):
+        v = Version(version)
+        v.epoch = None
+        prefix = '%s_%s.orig' % (package, v)
+        ret = []
+        for entry in os.scandir(self.path):
+            if entry.name.startswith(prefix):
+                component = (entry.name[len(prefix):].split('.')[0] or None)
+                shutil.copy(entry.path, target_dir)
+                if components is None or component in components:
+                    ret.append(os.path.join(target_dir, entry.name))
+        if ret:
+            return ret
+        raise PackageVersionNotPresent(package, version, self)
+
+    def get_recent_versions(self, package, since_version=None):
+        return []
+
+    def get_latest_version(self, package, version):
+        return None
