@@ -18,34 +18,46 @@
 #    along with bzr-builddeb; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+from debian.changelog import Version
+import os
 import re
+import subprocess
+import tempfile
 
 from .... import osutils
 from ....branch import (
     Branch,
     BranchWriteLockResult,
     )
-from ....bzr import branch as bzr_branch
 from ....errors import (
+    BzrError,
     GhostRevisionsHaveNoRevno,
     InvalidRevisionId,
-    InvalidRevisionSpec,
     NoSuchRevision,
     NoSuchTag,
+    RevisionNotPresent,
     UnsupportedOperation,
     )
-from ....lock import _RelockDebugMixin, LogicalLockResult
+from ..repack_tarball import get_filetype, repack_tarball
 from ....revision import NULL_REVISION
 from ....revisionspec import RevisionSpec
-from ....sixish import text_type
-from ....trace import note
+from ....trace import note, mutter
+from ....tree import Tree
+
+try:
+    from ....revisionspec import InvalidRevisionSpec
+except ImportError:  # Breezy < 3.2
+    from ....errors import InvalidRevisionSpec
 
 from ..errors import (
     MultipleUpstreamTarballsNotSupported,
-    PackageVersionNotPresent,
     )
 from .. import gettext
-from . import UpstreamSource
+from . import (
+    UpstreamSource,
+    PackageVersionNotPresent,
+    new_tarball_name,
+    )
 from ....export import (
     export,
     )
@@ -53,22 +65,30 @@ from ....export import (
 
 def upstream_tag_to_version(tag_name, package=None):
     """Take a tag name and return the upstream version, or None."""
-    assert isinstance(tag_name, text_type)
+    if tag_name.endswith('-release'):
+        tag_name = tag_name[:-len('-release')]
+    if tag_name.startswith("release-"):
+        tag_name = tag_name[len("release-"):]
+    if tag_name[0] == "v" and tag_name[1].isdigit():
+        tag_name = tag_name[1:]
     if (package is not None and (
           tag_name.startswith("%s-" % package) or
           tag_name.startswith("%s_" % package))):
-        return tag_name[len(package)+1:]
-    if tag_name.startswith("release-"):
-        return tag_name[len("release-"):]
-    if tag_name[0] == "v" and tag_name[1].isdigit():
-        return tag_name[1:]
+        tag_name = tag_name[len(package)+1:]
+    if '_' in tag_name and '.' not in tag_name:
+        tag_name = tag_name.replace('_', '.')
+    if tag_name.count('_') == 1 and tag_name.startswith('0.'):
+        # This is a style commonly used for perl packages.
+        # Most debian packages seem to just drop the underscore.
+        tag_name = tag_name.replace('_', '')
     if all([c.isdigit() or c in (".", "~") for c in tag_name]):
         return tag_name
     return None
 
 
-def _upstream_branch_version(revhistory, upstream_revision, reverse_tag_dict, package,
-                            previous_version, add_rev):
+def _upstream_branch_version(
+        revhistory, upstream_revision, reverse_tag_dict, package,
+        previous_version, add_rev):
     """Determine the version string of an upstream branch.
 
     The upstream version is determined from the most recent tag
@@ -83,7 +103,8 @@ def _upstream_branch_version(revhistory, upstream_revision, reverse_tag_dict, pa
     :param reverse_tag_dict: Reverse tag dictionary (revid -> list of tags)
     :param package: Name of package.
     :param previous_version: Previous upstream version in debian changelog.
-    :param add_rev: Function that can add a revision suffix to a version string.
+    :param add_rev: Function that can add a revision suffix to a version
+        string.
     :return: Name of the upstream revision.
     """
     if upstream_revision == NULL_REVISION:
@@ -106,6 +127,8 @@ def _upstream_branch_version(revhistory, upstream_revision, reverse_tag_dict, pa
     except RevisionNotPresent:
         # Ghost revision somewhere on mainline.
         pass
+    if previous_version is None:
+        return None
     return add_rev(str(previous_version), upstream_revision)
 
 
@@ -151,12 +174,14 @@ def extract_svn_revno(rev):
             return svn_revno
 
 
-def upstream_version_add_revision(upstream_branch, version_string, revid):
+def upstream_version_add_revision(
+        upstream_branch, version_string, revid, sep='+'):
     """Update the revision in a upstream version string.
 
     :param branch: Branch in which the revision can be found
     :param version_string: Original version string
     :param revid: Revision id of the revision
+    :param sep: Separator to use when adding snapshot
     """
     try:
         revno = upstream_branch.revision_id_to_dotted_revno(revid)
@@ -176,11 +201,11 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
         gitdate = osutils.format_date(
             rev.timestamp, rev.timezone, date_fmt='%Y%m%d', show_offset=False)
 
-    m = re.match(r"^(.*)([\+~])git(\d{8})\.([a-f0-9]{7})$", version_string)
+    m = re.match(r"^(.*)([\+~-])git(\d{8})\.([a-f0-9]{7})$", version_string)
     if m and gitid:
         return "%s%sgit%s.%s" % (m.group(1), m.group(2), gitdate, gitid)
 
-    m = re.match(r"^(.*)([\+~])git(\d{8})\.(\d+)\.([a-f0-9]{7})$",
+    m = re.match(r"^(.*)([\+~-])git(\d{8})\.(\d+)\.([a-f0-9]{7})$",
                  version_string)
     if m and gitid:
         if gitdate == m.group(3):
@@ -190,7 +215,7 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
         return "%s%sgit%s.%d.%s" % (
             m.group(1), m.group(2), gitdate, snapshot, gitid)
 
-    m = re.match(r"^(.*)([\+~])git(\d{8})$", version_string)
+    m = re.match(r"^(.*)([\+~-])git(\d{8})$", version_string)
     if m and gitid:
         return "%s%sgit%s" % (m.group(1), m.group(2), gitdate)
 
@@ -202,11 +227,11 @@ def upstream_version_add_revision(upstream_branch, version_string, revid):
         return "%s%ssvn%d" % (m.group(1), m.group(2), svn_revno)
 
     if svn_revno:
-        return "%s+svn%d" % (version_string, svn_revno)
+        return "%s%ssvn%d" % (version_string, sep, svn_revno)
     elif gitid:
-        return "%s+git%s.%s" % (version_string, gitdate, gitid)
+        return "%s%sgit%s.%s" % (version_string, sep, gitdate, gitid)
     else:
-        return "%s+bzr%s" % (version_string, revno_str)
+        return "%s%sbzr%s" % (version_string, sep, revno_str)
 
 
 def get_snapshot_revision(upstream_version):
@@ -242,25 +267,34 @@ def get_snapshot_revision(upstream_version):
 
 
 def upstream_branch_version(upstream_branch, upstream_revision, package,
-        previous_version):
+                            previous_version=None):
     """Determine the version string for a revision in an upstream branch.
 
     :param upstream_branch: The upstream branch object
     :param upstream_revision: The revision id of the upstream revision
     :param package: The name of the package
-    :param previous_version: The previous upstream version string
+    :param previous_version: The previous upstream version string (optional)
     :return: Upstream version string for `upstream_revision`.
     """
     graph = upstream_branch.repository.get_graph()
-    previous_revision = get_snapshot_revision(previous_version)
+    if previous_version is not None:
+        previous_revision = get_snapshot_revision(previous_version)
+    else:
+        previous_revision = None
+    stop_revids = None
     if previous_revision is not None:
         previous_revspec = RevisionSpec.from_string(previous_revision)
-        previous_revno, previous_revid = previous_revspec.in_history(upstream_branch)
-        # Trim revision history - we don't care about any revisions
-        # before the revision of the previous version
-        stop_revids = [previous_revid]
-    else:
-        stop_revids = None
+        try:
+            previous_revno, previous_revid = previous_revspec.in_history(
+                upstream_branch)
+        except InvalidRevisionSpec as e:
+            # Odd - the revision mentioned in the old version doesn't exist.
+            mutter('Unable to find old upstream version %s (%s): %s',
+                   previous_version, previous_revision, e)
+        else:
+            # Trim revision history - we don't care about any revisions
+            # before the revision of the previous version
+            stop_revids = [previous_revid]
     revhistory = graph.iter_lefthand_ancestry(upstream_revision, stop_revids)
     return _upstream_branch_version(
             revhistory, upstream_revision,
@@ -288,90 +322,105 @@ def get_export_upstream_revision(config=None, version=None):
     return rev
 
 
+def git_snapshot_data_from_version(version):
+    git_id = None
+    date = None
+    if "+git" in version or "~git" in version or "-git" in version:
+        m = re.match(
+            ".*[~+-]git([0-9]{4})([0-9]{2})([0-9]{2})\\.([0-9a-f]{7}).*",
+            version)
+        if not m:
+            m = re.match(
+                ".*[~+-]git([0-9]{4})([0-9]{2})([0-9]{2})\\.[0-9+]\\.([0-9a-f]{7}).*",
+                version)
+        if m:
+            git_id = m.group(4)
+            date = "%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
+        else:
+            m = re.match(".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2}).*", version)
+            if m:
+                date = "%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
+    return (git_id, date)
+
+
 def guess_upstream_revspec(package, version):
     """Guess revspecs matching an upstream version string."""
     if "+bzr" in version or "~bzr" in version:
         yield "revno:%s" % re.match(".*[~+]bzr(\\d+).*", version).group(1)
-    if "+git" in version or "~git" in version:
-        m = re.match(
-            ".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2})\\.([0-9a-f]{7}).*",
-            version)
-        if not m:
-            m = re.match(
-                ".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2})\\.[0-9+]\\.([0-9a-f]{7}).*",
-                version)
-        if m:
-            yield "git:%s" % m.group(4)
-            yield "date:%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
-        else:
-            m = re.match(".*[~+]git([0-9]{4})([0-9]{2})([0-9]{2}).*", version)
-            if m:
-                yield "date:%s-%s-%s" % (m.group(1), m.group(2), m.group(3))
-
+    (git_id, git_date) = git_snapshot_data_from_version(version)
+    if git_id:
+        yield "git:%s" % git_id
+    if git_date:
+        yield "date:%s" % git_date
     yield 'tag:%s' % version
     yield 'tag:%s-%s' % (package, version)
     yield 'tag:v%s' % version
     yield 'tag:v.%s' % version
 
 
-# TODO(jelmer): Move this into breezy.
-class MinimalMemoryBranch(bzr_branch.Branch, _RelockDebugMixin):
+try:
+    from breezy.memorybranch import MemoryBranch
+except ImportError:  # breezy < 3.1.1
+    from ....bzr import branch as bzr_branch
+    from ....lock import _RelockDebugMixin, LogicalLockResult
 
-    def __init__(self, repository, last_revision_info, tags):
-        from ....tag import DisabledTags, MemoryTags
-        self.repository = repository
-        self._last_revision_info = last_revision_info
-        self._revision_history_cache = None
-        if tags is not None:
-            self.tags = MemoryTags(tags)
-        else:
-            self.tags = DisabledTags(self)
-        self._partial_revision_history_cache = []
-        self._last_revision_info_cache = None
-        self._revision_id_to_revno_cache = None
-        self._partial_revision_id_to_revno_cache = {}
-        self._partial_revision_history_cache = []
+    class MemoryBranch(bzr_branch.Branch, _RelockDebugMixin):
 
-    def lock_read(self):
-        self.repository.lock_read()
-        return LogicalLockResult(self.unlock)
-
-    def lock_write(self, token=None):
-        self.repository.lock_write()
-        return BranchWriteLockResult(self.unlock, None)
-
-    def unlock(self):
-        self.repository.unlock()
-
-    def last_revision_info(self):
-        return self._last_revision_info
-
-    def _gen_revision_history(self):
-        """Generate the revision history from last revision
-        """
-        last_revno, last_revision = self.last_revision_info()
-        self._extend_partial_history()
-        return list(reversed(self._partial_revision_history_cache))
-
-    def get_rev_id(self, revno, history=None):
-        """Find the revision id of the specified revno."""
-        with self.lock_read():
-            if revno == 0:
-                return NULL_REVISION
-            last_revno, last_revid = self.last_revision_info()
-            if revno == last_revno:
-                return last_revid
-            if last_revno is None:
-                self._extend_partial_history()
-                return self._partial_revision_history_cache[
-                        len(self._partial_revision_history_cache) - revno]
+        def __init__(self, repository, last_revision_info, tags):
+            from ....tag import DisabledTags, MemoryTags
+            self.repository = repository
+            self._last_revision_info = last_revision_info
+            self._revision_history_cache = None
+            if tags is not None:
+                self.tags = MemoryTags(tags)
             else:
-                if revno <= 0 or revno > last_revno:
-                    raise NoSuchRevision(self, revno)
-                distance_from_last = last_revno - revno
-                if len(self._partial_revision_history_cache) <= distance_from_last:
-                    self._extend_partial_history(distance_from_last)
-                return self._partial_revision_history_cache[distance_from_last]
+                self.tags = DisabledTags(self)
+            self._partial_revision_history_cache = []
+            self._last_revision_info_cache = None
+            self._revision_id_to_revno_cache = None
+            self._partial_revision_id_to_revno_cache = {}
+            self._partial_revision_history_cache = []
+
+        def lock_read(self):
+            self.repository.lock_read()
+            return LogicalLockResult(self.unlock)
+
+        def lock_write(self, token=None):
+            self.repository.lock_write()
+            return BranchWriteLockResult(self.unlock, None)
+
+        def unlock(self):
+            self.repository.unlock()
+
+        def last_revision_info(self):
+            return self._last_revision_info
+
+        def _gen_revision_history(self):
+            """Generate the revision history from last revision
+            """
+            last_revno, last_revision = self.last_revision_info()
+            self._extend_partial_history()
+            return list(reversed(self._partial_revision_history_cache))
+
+        def get_rev_id(self, revno, history=None):
+            """Find the revision id of the specified revno."""
+            with self.lock_read():
+                if revno == 0:
+                    return NULL_REVISION
+                last_revno, last_revid = self.last_revision_info()
+                if revno == last_revno:
+                    return last_revid
+                if last_revno is None:
+                    self._extend_partial_history()
+                    return self._partial_revision_history_cache[
+                            len(self._partial_revision_history_cache) - revno]
+                else:
+                    if revno <= 0 or revno > last_revno:
+                        raise NoSuchRevision(self, revno)
+                    distance_from_last = last_revno - revno
+                    if len(self._partial_revision_history_cache) <= distance_from_last:
+                        self._extend_partial_history(distance_from_last)
+                    return self._partial_revision_history_cache[distance_from_last]
 
 
 class UpstreamBranchSource(UpstreamSource):
@@ -382,18 +431,22 @@ class UpstreamBranchSource(UpstreamSource):
     """
 
     def __init__(self, upstream_branch, upstream_revision_map=None,
-                 config=None, actual_branch=None):
+                 config=None, actual_branch=None, create_dist=None,
+                 other_repository=None, snapshot=True):
         self.upstream_branch = upstream_branch
         self._actual_branch = actual_branch or upstream_branch
+        self.create_dist = create_dist
         self.config = config
-        if upstream_revision_map is None:
-            self.upstream_revision_map = {}
-        else:
-            self.upstream_revision_map = upstream_revision_map
+        self.snapshot = snapshot
+        self.other_repository = other_repository
+        self.upstream_revision_map = {}
+        if upstream_revision_map is not None:
+            self.upstream_revision_map.update(upstream_revision_map.items())
 
     @classmethod
     def from_branch(cls, upstream_branch, upstream_revision_map=None,
-                    config=None, local_dir=None):
+                    config=None, local_dir=None, create_dist=None,
+                    snapshot=True):
         """Create a new upstream branch source from a branch.
 
         This will optionally fetch into a local directory.
@@ -410,13 +463,14 @@ class UpstreamBranchSource(UpstreamSource):
                 last_revision = upstream_branch.last_revision()
             local_repository.fetch(
                 upstream_branch.repository, revision_id=last_revision)
-            upstream_branch = MinimalMemoryBranch(
+            upstream_branch = MemoryBranch(
                 local_repository, (last_revno, last_revision),
                 upstream_branch.tags.get_tag_dict())
         return cls(
             upstream_branch=upstream_branch,
             upstream_revision_map=upstream_revision_map, config=config,
-            actual_branch=actual_branch)
+            actual_branch=actual_branch, create_dist=create_dist,
+            snapshot=snapshot)
 
     def version_as_revision(self, package, version, tarballs=None):
         if version in self.upstream_revision_map:
@@ -459,8 +513,16 @@ class UpstreamBranchSource(UpstreamSource):
             return True
 
     def get_latest_version(self, package, current_version):
-        return self.get_version(
-            package, current_version, self.upstream_branch.last_revision())
+        if self.snapshot:
+            revid = self.upstream_branch.last_revision()
+            version = self.get_version(package, current_version, revid)
+            self.upstream_revision_map[version] = 'revid:%s' % revid.decode('utf-8')
+            return version
+        else:
+            versions = list(self.get_recent_versions(package, current_version))
+            if not versions:
+                return None
+            return versions[-1]
 
     def get_recent_versions(self, package, since_version=None):
         versions = []
@@ -493,10 +555,25 @@ class UpstreamBranchSource(UpstreamSource):
                 revid = self.version_as_revision(package, version)
                 if revid is None:
                     raise PackageVersionNotPresent(package, version, self)
+            if self.other_repository is not None:
+                try:
+                    rev_tree = self.other_repository.revision_tree(revid)
+                except NoSuchRevision:
+                    rev_tree = None
+            else:
+                rev_tree = None
+            if rev_tree is None:
+                rev_tree = self.upstream_branch.repository.revision_tree(revid)
+            if self.create_dist is not None:
+                with tempfile.TemporaryDirectory() as td:
+                    fn = self.create_dist(rev_tree, package, version, td)
+                    if fn:
+                        nfn = new_tarball_name(package, version, fn)
+                        repack_tarball(os.path.join(td, fn), nfn, target_dir)
+                        return [os.path.join(target_dir, nfn)]
+            tarball_base = "%s-%s" % (package, version)
             target_filename = self._tarball_path(
                 package, version, None, target_dir)
-            tarball_base = "%s-%s" % (package, version)
-            rev_tree = self.upstream_branch.repository.revision_tree(revid)
             try:
                 export(rev_tree, target_filename, 'tgz', tarball_base)
             except UnsupportedOperation as e:
@@ -517,10 +594,14 @@ class LazyUpstreamBranchSource(UpstreamBranchSource):
     """
 
     def __init__(self, upstream_branch_url, upstream_revision_map=None,
-                 config=None):
+                 config=None, create_dist=None, other_repository=None,
+                 snapshot=True):
         self.upstream_branch_url = upstream_branch_url
+        self.snapshot = snapshot
         self._upstream_branch = None
         self.config = config
+        self.create_dist = create_dist
+        self.other_repository = other_repository
         if upstream_revision_map is None:
             self.upstream_revision_map = {}
         else:
@@ -537,21 +618,54 @@ class LazyUpstreamBranchSource(UpstreamBranchSource):
             self.__class__.__name__, self.upstream_branch_url)
 
 
-class LocalUpstreamBranchSource(UpstreamBranchSource):
-    """Upstream branch source in a local branch."""
+class DistCommandFailed(BzrError):
 
-    def __init__(self, local_branch):
-        self.local_branch = local_branch
-        self.upstream_revision_map = {}
-        self.config = None
+    _fmt = "Dist command failed to produce a tarball: %(error)s"
 
-    @classmethod
-    def from_controldir(cls, controldir):
-        return cls(controldir.open_branch('upstream'))
+    def __init__(self, error):
+        super(DistCommandFailed, self).__init__(error=error)
 
-    @property
-    def upstream_branch(self):
-        return self.local_branch
 
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self.local_branch)
+def run_dist_command(
+        rev_tree: Tree, package: str, version: Version, target_dir: str,
+        dist_command: str) -> bool:
+    with tempfile.TemporaryDirectory() as td:
+        package_dir = os.path.join(td, package)
+        export(rev_tree, package_dir, 'dir')
+        existing_files = os.listdir(package_dir)
+        env = dict(os.environ.items())
+        env['PACKAGE'] = package
+        env['VERSION'] = version
+        note('Running dist command: %s', dist_command)
+        try:
+            subprocess.check_call(
+                dist_command, env=env, cwd=package_dir, shell=True)
+        except subprocess.CalledProcessError as e:
+            if e.returncode == 2:
+                return None
+            raise DistCommandFailed(str(e))
+        new_files = os.listdir(package_dir)
+        diff_files = set(new_files) - set(existing_files)
+        diff = [n for n in diff_files if get_filetype(n) is not None]
+        if len(diff) == 1:
+            note('Found tarball %s in package directory.', diff[0])
+            os.rename(
+                os.path.join(package_dir, diff[0]),
+                os.path.join(target_dir, diff[0]))
+            return diff[0]
+        if 'dist' in diff_files:
+            for entry in os.scandir(os.path.join(package_dir, 'dist')):
+                if get_filetype(entry.name) is not None:
+                    note('Found tarball %s in dist directory.', entry.name)
+                    os.rename(entry.path, os.path.join(target_dir, entry.name))
+                    return entry.name
+            note('No tarballs found in dist directory.')
+        diff = set(os.listdir(td)) - set([package])
+        if len(diff) == 1:
+            fn = diff.pop()
+            note('Found tarball %s in parent directory.', fn)
+            os.rename(
+                os.path.join(td, fn),
+                os.path.join(target_dir, fn))
+            return fn
+    raise DistCommandFailed('no tarball created')
