@@ -21,10 +21,7 @@
 from __future__ import absolute_import
 
 import errno
-try:
-    import hashlib as md5
-except ImportError:
-    import md5
+import hashlib
 import signal
 import shutil
 import subprocess
@@ -34,16 +31,19 @@ import re
 
 from debian import deb822
 from debian.changelog import Changelog, ChangelogParseError
+from debian.copyright import Copyright, NotMachineReadableError
+
+from debmutate.changelog import strip_changelog_message
 
 from ... import (
     bugtracker,
     errors,
+    osutils,
     urlutils,
-    version_info as bzr_version_info,
     )
-from ...sixish import text_type
 from ...trace import (
     mutter,
+    note,
     warning,
     )
 from ...transport import (
@@ -51,11 +51,7 @@ from ...transport import (
     get_transport,
     )
 from . import (
-    default_conf,
-    local_conf,
     global_conf,
-    new_conf,
-    new_local_conf,
     )
 from .config import (
     DebBuildConfig,
@@ -68,11 +64,17 @@ from .errors import (
     AddChangelogError,
     InconsistentSourceFormatError,
     NoPreviousUpload,
-    TarFailed,
     UnableToFindPreviousUpload,
     UnparseableChangelog,
     )
 
+BUILDDEB_DIR = '.bzr-builddeb'
+
+NEW_LOCAL_CONF = 'debian/local.conf.local'
+NEW_CONF = 'debian/bzr-builddeb.conf'
+DEFAULT_CONF = os.path.join(BUILDDEB_DIR, 'default.conf')
+LOCAL_CONF = os.path.join(BUILDDEB_DIR, 'local.conf')
+
 
 class MissingChangelogError(BzrError):
     _fmt = 'Could not find changelog at %(location)s in tree.'
@@ -84,12 +86,14 @@ class MissingChangelogError(BzrError):
 _DEBIAN_RELEASES = None
 _UBUNTU_RELEASES = None
 
+
 def _get_release_names():
     global _DEBIAN_RELEASES, _UBUNTU_RELEASES
     try:
         from distro_info import DebianDistroInfo, UbuntuDistroInfo
     except ImportError:
-        warning("distro_info not available. Unable to retrieve current "
+        warning(
+            "distro_info not available. Unable to retrieve current "
             "list of releases.")
         _DEBIAN_RELEASES = []
         _UBUNTU_RELEASES = []
@@ -106,24 +110,26 @@ def debian_releases():
         _get_release_names()
     return _DEBIAN_RELEASES
 
+
 def ubuntu_releases():
     if _UBUNTU_RELEASES is None:
         _get_release_names()
     return _UBUNTU_RELEASES
 
+
 DEBIAN_POCKETS = ('', '-security', '-proposed-updates', '-backports')
 UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
 
 
 def safe_decode(s):
     """Decode a string into a Unicode value."""
-    if isinstance(s, text_type): # Already unicode
+    if isinstance(s, str):  # Already unicode
         mutter('safe_decode() called on an already-decoded string: %r' % (s,))
         return s
     try:
         return s.decode('utf-8')
     except UnicodeDecodeError as e:
-        mutter('safe_decode(%r) falling back to iso-8859-1' % (s,))
+        mutter('safe_decode(%r) falling back to iso-8859-1: %s' % (s, e))
         # TODO: Looking at BeautifulSoup it seems to use 'chardet' to try to
         #       guess the encoding of a given text stream. We might want to
         #       take a closer look at that.
@@ -148,10 +154,15 @@ def recursive_copy(fromdir, todir):
                 os.mkdir(tosubdir)
             recursive_copy(path, tosubdir)
         else:
-            shutil.copy(path, todir)
+            # Python 3 has a follow_symlinks argument to shutil.copy, but
+            # Python 2 does not...
+            if os.path.islink(path):
+                os.symlink(os.readlink(path), os.path.join(todir, entry))
+            else:
+                shutil.copy(path, todir)
 
 
-def find_changelog(t, merge=False, max_blocks=1):
+def find_changelog(t, subpath='', merge=False, max_blocks=1):
     """Find the changelog in the given tree.
 
     First looks for 'debian/changelog'. If "merge" is true will also
@@ -160,7 +171,7 @@ def find_changelog(t, merge=False, max_blocks=1):
     The returned changelog is created with 'allow_empty_author=True'
     as some people do this but still want to build.
     'max_blocks' defaults to 1 to try and prevent old broken
-    changelog entries from causing the command to fail, 
+    changelog entries from causing the command to fail.
 
     "top_level" is a subset of "merge" mode. It indicates that the
     '.bzr' dir is at the same level as 'changelog' etc., rather
@@ -177,12 +188,12 @@ def find_changelog(t, merge=False, max_blocks=1):
     """
     top_level = False
     with t.lock_read():
-        changelog_file = 'debian/changelog'
+        changelog_file = osutils.pathjoin(subpath, 'debian/changelog')
         if not t.has_filename(changelog_file):
-            checked_files = ['debian/changelog']
+            checked_files = [changelog_file]
             if merge:
                 # Assume LarstiQ's layout (.bzr in debian/)
-                changelog_file = 'changelog'
+                changelog_file = osutils.pathjoin(subpath, 'changelog')
                 top_level = True
                 if not t.has_filename(changelog_file):
                     checked_files.append(changelog_file)
@@ -193,13 +204,14 @@ def find_changelog(t, merge=False, max_blocks=1):
                 if getattr(t, "abspath", None):
                     checked_files = [t.abspath(f) for f in checked_files]
                 raise MissingChangelogError(" or ".join(checked_files))
-        elif merge and t.has_filename('changelog'):
+        elif merge and t.has_filename(osutils.pathjoin(subpath, 'changelog')):
             # If it is a "top_level" package and debian is a symlink to
             # "." then it will have found debian/changelog. Try and detect
             # this.
-            if (t.is_versioned('debian') and
-                t.kind('debian') == 'symlink' and
-                t.get_symlink_target('debian') == '.'):
+            debian_file = osutils.pathjoin(subpath, 'debian')
+            if (t.is_versioned(debian_file) and
+                    t.kind(debian_file) == 'symlink' and
+                    t.get_symlink_target(debian_file) == '.'):
                 changelog_file = 'changelog'
                 top_level = True
         mutter("Using '%s' to get package information", changelog_file)
@@ -208,41 +220,13 @@ def find_changelog(t, merge=False, max_blocks=1):
         contents = t.get_file_text(changelog_file)
     changelog = Changelog()
     try:
-        changelog.parse_changelog(contents, max_blocks=max_blocks, allow_empty_author=True)
+        changelog.parse_changelog(
+            contents, max_blocks=max_blocks, allow_empty_author=True)
     except ChangelogParseError as e:
         raise UnparseableChangelog(str(e))
     return changelog, top_level
 
 
-def strip_changelog_message(changes):
-    """Strip a changelog message like debcommit does.
-
-    Takes a list of changes from a changelog entry and applies a transformation
-    so the message is well formatted for a commit message.
-
-    :param changes: a list of lines from the changelog entry
-    :return: another list of lines with blank lines stripped from the start
-        and the spaces the start of the lines split if there is only one logical
-        entry.
-    """
-    if not changes:
-        return changes
-    while changes and changes[-1] == '':
-        changes.pop()
-    while changes and changes[0] == '':
-        changes.pop(0)
-
-    whitespace_column_re = re.compile(r'  |\t')
-    changes = [whitespace_column_re.sub('', line, 1) for line in changes]
-
-    leader_re = re.compile(r'[ \t]*[*+-] ')
-    count = len([l for l in changes if leader_re.match(l)])
-    if count == 1:
-        return [leader_re.sub('', line, 1).lstrip() for line in changes]
-    else:
-        return changes
-
-
 def tarball_name(package, version, component=None, format=None):
     """Return the name of the .orig.tar.gz for the given package and version.
 
@@ -278,11 +262,13 @@ def suite_to_distribution(suite):
         return "debian"
     if suite in all_ubuntu:
         return "ubuntu"
+    if suite == 'kali' or suite.startswith('kali-'):
+        return "kali"
     return None
 
 
 def lookup_distribution(distribution_or_suite):
-    """Get the distribution name based on a distribtion or suite name.
+    """Get the distribution name based on a distribution or suite name.
 
     :param distribution_or_suite: a string that is either the name of
         a distribution or a suite.
@@ -299,18 +285,16 @@ def md5sum_filename(filename):
     :param filename: Path of the file to checksum
     :return: MD5 Checksum as hex digest
     """
-    m = md5.md5()
-    f = open(filename, 'rb')
-    try:
+    m = hashlib.md5()
+    with open(filename, 'rb') as f:
         for line in f:
             m.update(line)
-    finally:
-        f.close()
     return m.hexdigest()
 
 
 def move_file_if_different(source, target, md5sum):
-    """Overwrite a file if its new contents would be different from the current contents.
+    """Overwrite a file if its new contents would be different from the current
+    contents.
 
     :param source: Path of the source file
     :param target: Path of the target file
@@ -326,12 +310,13 @@ def move_file_if_different(source, target, md5sum):
 
 
 def write_if_different(contents, target):
-    """(Over)write a file with `contents` if they are different from its current content.
+    """(Over)write a file with `contents` if they are different from its
+    current content.
 
     :param contents: The contents to write, as a string
     :param target: Path of the target file
     """
-    md5sum = md5.md5()
+    md5sum = hashlib.md5()
     md5sum.update(contents)
     fd, temp_path = tempfile.mkstemp("builddeb-rename-")
     fobj = os.fdopen(fd, "wb")
@@ -351,8 +336,7 @@ def _download_part(name, base_transport, target_dir, md5sum):
     f_t = base_transport
     if part_base_dir != '':
         f_t = base_transport.clone(part_base_dir)
-    f_f = f_t.get(part_path)
-    try:
+    with f_t.get(part_path) as f_f:
         target_path = os.path.join(target_dir, part_path)
         fd, temp_path = tempfile.mkstemp(prefix="builddeb-")
         fobj = os.fdopen(fd, "wb")
@@ -365,8 +349,6 @@ def _download_part(name, base_transport, target_dir, md5sum):
         finally:
             if os.path.exists(temp_path):
                 os.unlink(temp_path)
-    finally:
-        f_f.close()
 
 
 def open_file(url):
@@ -390,6 +372,7 @@ def open_file_via_transport(filename, transport):
     """Open a file using the transport, follow redirects as necessary."""
     def open_file(transport):
         return transport.get(filename)
+
     def follow_redirection(transport, e, redirection_notice):
         mutter(redirection_notice)
         _filename, redirected_transport = open_transport(e.target)
@@ -412,7 +395,8 @@ def _dget(cls, dsc_location, target_dir):
     if not os.path.isdir(target_dir):
         raise errors.NotADirectory(target_dir)
     path, dsc_t = open_transport(dsc_location)
-    dsc_contents = open_file_via_transport(path, dsc_t).read()
+    with open_file_via_transport(path, dsc_t) as f:
+        dsc_contents = f.read()
     dsc = cls(dsc_contents)
     for file_details in dsc['files']:
         name = file_details['name']
@@ -442,14 +426,15 @@ def find_bugs_fixed(changes, branch, _lplib=None):
 
     :param changes: A list of the contents of the changelog entry.
     :param branch: Bazaar branch associated with the package
-    :return: String with bugs closed, as appropriate for a Bazaar "bugs" revision 
-        property.
+    :return: String with bugs closed, as appropriate for a Bazaar "bugs"
+        revision property.
     """
     if _lplib is None:
         from . import launchpad as _lplib
     bugs = []
     for change in changes:
-        for match in re.finditer("closes:\\s*(?:bug)?\\#?\\s?\\d+"
+        for match in re.finditer(
+                "closes:\\s*(?:bug)?\\#?\\s?\\d+"
                 "(?:,\\s*(?:bug)?\\#?\\s?\\d+)*", change,
                 re.IGNORECASE):
             closes_list = match.group(0)
@@ -460,15 +445,16 @@ def find_bugs_fixed(changes, branch, _lplib=None):
                 if len(lp_bugs) == 1:
                     bug_url = bugtracker.get_bug_url("lp", branch, lp_bugs[0])
                     bugs.append(bug_url + " fixed")
-        for match in re.finditer("lp:\\s+\\#\\d+(?:,\\s*\\#\\d+)*",
-                change, re.IGNORECASE):
+        for match in re.finditer(
+                "lp:\\s+\\#\\d+(?:,\\s*\\#\\d+)*", change, re.IGNORECASE):
             closes_list = match.group(0)
             for match in re.finditer("\\d+", closes_list):
                 bug_url = bugtracker.get_bug_url("lp", branch, match.group(0))
                 bugs.append(bug_url + " fixed")
                 deb_bugs = _lplib.debian_bugs_for_ubuntu_bug(match.group(0))
                 if len(deb_bugs) == 1:
-                    bug_url = bugtracker.get_bug_url("deb", branch, deb_bugs[0])
+                    bug_url = bugtracker.get_bug_url(
+                        "deb", branch, deb_bugs[0])
                     bugs.append(bug_url + " fixed")
     return bugs
 
@@ -501,10 +487,11 @@ def find_thanks(changes):
     :param changes: String with the contents of the changelog entry
     :return: List of people thanked, optionally including email address.
     """
-    thanks_re = re.compile(r"[tT]hank(?:(?:s)|(?:you))(?:\s*to)?"
-            "((?:\\s+(?:(?:\\w\\.)|(?:\\w+(?:-\\w+)*)))+"
-            "(?:\\s+<[^@>]+@[^@>]+>)?)",
-            re.UNICODE)
+    thanks_re = re.compile(
+        r"[tT]hank(?:(?:s)|(?:you))(?:\s*to)?"
+        "((?:\\s+(?:(?:\\w\\.)|(?:\\w+(?:-\\w+)*)))+"
+        "(?:\\s+<[^@>]+@[^@>]+>)?)",
+        re.UNICODE)
     thanks = []
     changes_str = safe_decode(" ".join(changes))
     for match in thanks_re.finditer(changes_str):
@@ -567,41 +554,48 @@ def subprocess_setup():
     signal.signal(signal.SIGPIPE, signal.SIG_DFL)
 
 
-def debuild_config(tree):
+def debuild_config(tree, subpath):
     """Obtain the Debuild configuration object.
 
     :param tree: A Tree object, can be a WorkingTree or RevisionTree.
     """
     config_files = []
     user_config = None
+    if subpath in ('.', None):
+        subpath = ''
+    new_local_conf = osutils.pathjoin(subpath, NEW_LOCAL_CONF)
+    local_conf = osutils.pathjoin(subpath, LOCAL_CONF)
+    new_conf = osutils.pathjoin(subpath, NEW_CONF)
+    default_conf = osutils.pathjoin(subpath, DEFAULT_CONF)
+
     if tree.has_filename(new_local_conf):
         if not tree.is_versioned(new_local_conf):
-            config_files.append((tree.get_file(new_local_conf), True,
-                        "local.conf"))
+            config_files.append(
+                (tree.get_file(new_local_conf), True, "local.conf"))
         else:
             warning('Not using configuration from %s as it is versioned.',
                     new_local_conf)
     if tree.has_filename(local_conf):
         if not tree.is_versioned(local_conf):
-            config_files.append((tree.get_file(local_conf), True,
-                        "local.conf"))
+            config_files.append(
+                (tree.get_file(local_conf), True, "local.conf"))
         else:
             warning('Not using configuration from %s as it is versioned.',
                     local_conf)
     config_files.append((global_conf(), True))
     user_config = global_conf()
     if tree.is_versioned(new_conf):
-        config_files.append((tree.get_file(new_conf), False,
-                    "bzr-builddeb.conf"))
+        config_files.append(
+            (tree.get_file(new_conf), False, "bzr-builddeb.conf"))
     if tree.is_versioned(default_conf):
-        config_files.append((tree.get_file(default_conf), False,
-                    "default.conf"))
+        config_files.append(
+            (tree.get_file(default_conf), False, "default.conf"))
     config = DebBuildConfig(config_files, tree=tree)
     config.set_user_config(user_config)
     return config
 
 
-def find_previous_upload(tree, merge=False):
+def find_previous_upload(tree, subpath, merge=False):
     """Given a tree, find the previous upload to the distribution.
 
     When e.g. Ubuntu merges from Debian they want to build with
@@ -619,7 +613,7 @@ def find_previous_upload(tree, merge=False):
     Ubuntu.
     """
     try:
-        cl, top_level = find_changelog(tree, merge, max_blocks=None)
+        cl, top_level = find_changelog(tree, subpath, merge, max_blocks=None)
     except UnparseableChangelog:
         raise UnableToFindPreviousUpload()
     return changelog_find_previous_upload(cl)
@@ -640,21 +634,20 @@ def changelog_find_previous_upload(cl):
     elif current_target in all_ubuntu:
         match_targets = ubuntu_releases()
         if "-" in current_target:
-            match_targets += tuple([current_target.split("-", 1)[0]
-                + t for t in UBUNTU_POCKETS])
+            match_targets += tuple(
+                [current_target.split("-", 1)[0] + t for t in UBUNTU_POCKETS])
     else:
         # If we do not recognize the current target in order to apply special
         # rules to it, then just assume that only previous uploads to exactly
         # the same target count.
         match_targets = (current_target,)
-    previous_version = None
     for block in cl._blocks[1:]:
         if block.distributions.split(" ")[0] in match_targets:
             return block.version
     raise NoPreviousUpload(current_target)
 
 
-def tree_contains_upstream_source(tree):
+def tree_contains_upstream_source(tree, subpath=''):
     """Guess if the specified tree contains the upstream source.
 
     :param tree: A RevisionTree.
@@ -662,7 +655,7 @@ def tree_contains_upstream_source(tree):
         source. None if the tree is empty
     """
     present_files = set(
-        [f[0] for f in tree.list_files(recursive=False)
+        [f[0] for f in tree.list_files(recursive=False, from_dir=subpath)
          if f[1] == 'V'])
     if len(present_files) == 0:
         return None
@@ -671,13 +664,13 @@ def tree_contains_upstream_source(tree):
     return (len(present_files - packaging_files) > 0)
 
 
-def tree_get_source_format(tree):
+def tree_get_source_format(tree, subpath=''):
     """Retrieve the source format name from a package.
 
     :param path: Path to the package
     :return: String with package format
     """
-    filename = "debian/source/format"
+    filename = osutils.pathjoin(subpath, "debian/source/format")
     try:
         text = tree.get_file_text(filename)
     except IOError as e:
@@ -697,15 +690,16 @@ NATIVE_SOURCE_FORMATS = [FORMAT_3_0_NATIVE]
 NORMAL_SOURCE_FORMATS = [FORMAT_3_0_QUILT]
 
 
-def guess_build_type(tree, version, contains_upstream_source):
+def guess_build_type(tree, version, subpath='', contains_upstream_source=True):
     """Guess the build type based on the contents of a tree.
 
     :param tree: A `Tree` object.
     :param version: `Version` of the upload.
-    :param contains_upstream_source: Whether this branch contains the upstream source.
+    :param contains_upstream_source: Whether this branch contains the upstream
+        source.
     :return: A build_type value.
     """
-    source_format = tree_get_source_format(tree)
+    source_format = tree_get_source_format(tree, subpath)
     if source_format in NATIVE_SOURCE_FORMATS:
         format_native = True
     elif source_format in NORMAL_SOURCE_FORMATS:
@@ -713,19 +707,24 @@ def guess_build_type(tree, version, contains_upstream_source):
     else:
         format_native = None
 
-    # If the package doesn't have a debian revision then it must be native.
     if version is not None:
         version_native = (not version.debian_version)
     else:
         version_native = None
 
+    # If the package doesn't have a debian revision then it is very probably
+    # native, but it *could* be native.
     if type(version_native) is bool and type(format_native) is bool:
-        if version_native != format_native:
+        if version_native is True and format_native is False:
             raise InconsistentSourceFormatError(version_native, format_native)
+        if version_native is False and format_native is True:
+            warning(
+                'Version suggests non-native package, '
+                'but format is for native.')
 
     if version_native or format_native:
         return BUILD_TYPE_NATIVE
-    if contains_upstream_source == False:
+    if contains_upstream_source is False:
         # Default to merge mode if there's only a debian/ directory
         return BUILD_TYPE_MERGE
     else:
@@ -753,26 +752,65 @@ def component_from_orig_tarball(tarball_filename, package, version):
         # Extra component
         return base[1:]
     else:
-        raise ValueError("Invalid extra characters in tarball filename %s" %
-            tarball_filename)
+        raise ValueError(
+                "Invalid extra characters in tarball filename %s" %
+                tarball_filename)
 
 
-def extract_orig_tarball(tarball_filename, component, target, strip_components=None):
+class TarFailed(BzrError):
+    _fmt = ("There was an error executing tar to %(operation)s %(tarball)s: "
+            "%(error)s.")
+
+    def __init__(self, operation, tarball, error):
+        BzrError.__init__(
+            self, operation=operation, tarball=tarball, error=error)
+
+
+def needs_strip_components(tf):
+    top_level_directories = set()
+    for name in tf.getnames():
+        top_level_directories.add(name.split('/')[0])
+    return len(top_level_directories) == 1
+
+
+def extract_orig_tarball(tarball_filename, component, target,
+                         strip_components=None):
     """Extract an orig tarball.
 
     :param tarball: Path to the tarball
     :param component: Component name (or None for top-level)
     :param target: Target path
-    :param strip_components: Optional number of components to strip
     """
+    from tarfile import TarFile
     tar_args = ["tar"]
     if tarball_filename.endswith(".tar.bz2"):
         tar_args.append('xjf')
+        tf = TarFile.bz2open(tarball_filename)
     elif (tarball_filename.endswith(".tar.lzma") or
           tarball_filename.endswith(".tar.xz")):
         tar_args.append('xJf')
-    else:
+        tf = TarFile.xzopen(tarball_filename)
+    elif tarball_filename.endswith(".tar"):
+        tar_args.append('xf')
+        tf = TarFile.open(tarball_filename)
+    elif (tarball_filename.endswith(".tar.gz") or
+            tarball_filename.endswith(".tgz")):
+        tf = TarFile.gzopen(tarball_filename)
         tar_args.append('xzf')
+    else:
+        note('Unable to figure out type of %s, '
+             'assuming .tar.gz', tarball_filename)
+        tf = TarFile.gzopen(tarball_filename)
+        tar_args.append('xzf')
+
+    try:
+        if strip_components is None:
+            if needs_strip_components(tf):
+                strip_components = 1
+            else:
+                strip_components = 0
+    finally:
+        tf.close()
     if component is not None:
         target_path = os.path.join(target, component)
         os.mkdir(target_path)
@@ -780,11 +818,12 @@ def extract_orig_tarball(tarball_filename, component, target, strip_components=N
         target_path = target
     tar_args.extend([tarball_filename, "-C", target_path])
     if strip_components is not None:
-        tar_args.extend(["--strip-components", "1"])
-    proc = subprocess.Popen(tar_args, preexec_fn=subprocess_setup)
-    proc.communicate()
+        tar_args.extend(["--strip-components", str(strip_components)])
+    proc = subprocess.Popen(tar_args, preexec_fn=subprocess_setup,
+                            stderr=subprocess.PIPE)
+    (stdout, stderr) = proc.communicate()
     if proc.returncode != 0:
-        raise TarFailed("extract", tarball_filename)
+        raise TarFailed("extract", tarball_filename, error=stderr)
 
 
 def extract_orig_tarballs(tarballs, target, strip_components=None):
@@ -794,7 +833,8 @@ def extract_orig_tarballs(tarballs, target, strip_components=None):
     :param target: Target directory (must already exist)
     """
     for tarball_filename, component in tarballs:
-        extract_orig_tarball(tarball_filename, component, target,
+        extract_orig_tarball(
+            tarball_filename, component, target,
             strip_components=strip_components)
 
 
@@ -817,8 +857,7 @@ def changes_filename(package, version, arch):
     non_epoch_version = version.upstream_version
     if version.debian_version is not None:
         non_epoch_version += "-%s" % version.debian_version
-    return "%s_%s_%s.changes" % (package,
-            non_epoch_version, arch)
+    return "%s_%s_%s.changes" % (package, non_epoch_version, arch)
 
 
 def get_build_architecture():
@@ -828,3 +867,19 @@ def get_build_architecture():
     except subprocess.CalledProcessError as e:
         raise BzrError(
             "Could not find the build architecture: %s" % e)
+
+
+def get_files_excluded(tree, subpath='', top_level=False):
+    if top_level:
+        path = os.path.join(subpath, 'copyright')
+    else:
+        path = os.path.join(subpath, 'debian', 'copyright')
+    with tree.get_file(path) as f:
+        try:
+            copyright = Copyright(f, strict=False)
+        except NotMachineReadableError:
+            return []
+        try:
+            return copyright.header["Files-Excluded"].split()
+        except KeyError:
+            return []
